@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: systemd.pm 1670472 2015-03-31 20:34:45Z jfthomps $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -31,7 +31,7 @@ VCL::Module::OS::Linux::init::systemd.pm
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::Module::OS::Linux::init::systemd;
 
 # Specify the lib path using FindBin
@@ -42,7 +42,7 @@ use lib "$FindBin::Bin/../../../../..";
 use base qw(VCL::Module::OS::Linux::init);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -53,7 +53,7 @@ use diagnostics;
 
 use VCL::utils;
 
-##############################################################################
+###############################################################################
 
 =head1 CLASS VARIABLES
 
@@ -82,13 +82,13 @@ our $INIT_DAEMON_ORDER = 20;
 
 our @REQUIRED_COMMANDS = ('systemctl');
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 get_service_names
 
@@ -128,43 +128,7 @@ sub get_service_names {
 	return @service_names;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
-
-=head2 service_exists
-
- Parameters  : $service_name
- Returns     : boolean
- Description : Determines if a service controlled by systemd exists on the
-               computer.
-
-=cut
-
-sub service_exists {
-	my $self = shift;
-	if (ref($self) !~ /linux/i) {
-		notify($ERRORS{'CRITICAL'}, 0, "subroutine was called as a function, it must be called as a class method");
-		return;
-	}
-	my ($service_name) = @_;
-	if (!defined($service_name)) {
-		notify($ERRORS{'WARNING'}, 0, "service name argument was not supplied");
-		return;
-	}
-	
-	my $computer_node_name = $self->data->get_computer_node_name();
-	
-	my @service_names = $self->get_service_names();
-	if (grep { $_ eq $service_name } @service_names) {
-		notify($ERRORS{'DEBUG'}, 0, "$service_name service exists on $computer_node_name");
-		return 1;
-	}
-	else {
-		notify($ERRORS{'DEBUG'}, 0, "$service_name service does not exist on $computer_node_name");
-		return 0;
-	}
-}
-
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 service_running
 
@@ -212,7 +176,7 @@ sub service_running {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 service_enabled
 
@@ -260,7 +224,7 @@ sub service_enabled {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 enable_service
 
@@ -299,7 +263,7 @@ sub enable_service {
 		return;
 	}
 	elsif ($exit_status ne 0 || grep(/(failed)/i, @$output)) {
-		notify($ERRORS{'WARNING'}, 0, "failed to enable '$service_name' service on $computer_node_name, exit status: $exit_status, output:\n" . join("\n", @$output));
+		notify($ERRORS{'WARNING'}, 0, "failed to enable '$service_name' service on $computer_node_name, exit status: $exit_status, command:\n$command\noutput:\n" . join("\n", @$output));
 		return;
 	}
 	else {
@@ -309,7 +273,7 @@ sub enable_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 disable_service
 
@@ -357,7 +321,7 @@ sub disable_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 delete_service
 
@@ -384,10 +348,8 @@ sub delete_service {
 	my $computer_node_name = $self->data->get_computer_node_name();
 	
 	# Disable the service before deleting it
-	if ($self->service_exists($service_name)) {
-		$self->stop_service($service_name) || return;
-		$self->disable_service($service_name) || return;
-	}
+	$self->stop_service($service_name) || return;
+	$self->disable_service($service_name) || return;
 	
 	# Delete the service configuration file
 	my $service_file_path = "/lib/systemd/system/$service_name.service";
@@ -400,7 +362,7 @@ sub delete_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 start_service
 
@@ -450,7 +412,7 @@ sub start_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 stop_service
 
@@ -486,7 +448,13 @@ sub stop_service {
 	elsif (grep(/(Failed to issue method call|No such file)/i, @$output)) {
 		# Output if the service doesn't exist
 		# Failed to issue method call: Unit httpdx.service failed to load: No such file or directory.
-		notify($ERRORS{'WARNING'}, 0, "unable to stop '$service_name' service because it does not exist on $computer_node_name");
+		notify($ERRORS{'DEBUG'}, 0, "unable to stop '$service_name' service because it does not exist on $computer_node_name");
+		return 1;
+	}
+	elsif (grep(/(not loaded)/i, @$output)) {
+		# Output if the service isn't loaded
+		# Failed to stop ext_ssh.service: Unit ext_ssh.service not loaded.
+		notify($ERRORS{'DEBUG'}, 0, "unable to stop '$service_name' service because it is not loaded $computer_node_name");
 		return 1;
 	}
 	elsif ($exit_status ne 0 || grep(/(failed)/i, @$output)) {
@@ -500,7 +468,7 @@ sub stop_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 restart_service
 
@@ -550,7 +518,7 @@ sub restart_service {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 add_ext_sshd_service
 
@@ -570,7 +538,15 @@ sub add_ext_sshd_service {
 	
 	my $computer_node_name = $self->data->get_computer_node_name();
 	
-	my $sshd_service_file_path     = '/lib/systemd/system/sshd.service';
+	# Get the unit file path for the sshd service
+	# Do not automatically assume it is /lib/systemd/system/sshd.service
+	# https://issues.apache.org/jira/browse/VCL-989
+	my $sshd_service_file_path = $self->_get_service_unit_file_path('sshd');
+	if (!$sshd_service_file_path) {
+		$sshd_service_file_path = '/lib/systemd/system/sshd.service';
+	}
+	
+	# Hard-code the ext_sshd file path (intentional)
 	my $ext_sshd_service_file_path = '/lib/systemd/system/ext_sshd.service';
 	
 	# Get the contents of the sshd service configuration file already on the computer
@@ -581,22 +557,47 @@ sub add_ext_sshd_service {
 	}
 	
 	my $ext_sshd_service_file_contents = join("\n", @sshd_service_file_contents);
-	
+
 	# Replace: OpenSSH --> External OpenSSH
 	$ext_sshd_service_file_contents =~ s|(OpenSSH)|external $1|g;
 	
 	# Replace: sshd --> ext_sshd, exceptions:
 	# /bin/sshd
 	# /sshd_config
-	$ext_sshd_service_file_contents =~ s|(?<!bin/)sshd(?!_config)|ext_sshd|g;
+	$ext_sshd_service_file_contents =~ s|(?<!bin/)sshd(?!_config\|-keygen)|ext_sshd|g;
 	
-	# Replace: $OPTIONS --> -f /etc/ssh/external_sshd_config
-	$ext_sshd_service_file_contents =~ s|(ExecStart=.+)\s+\$OPTIONS|$1 -f /etc/ssh/external_sshd_config|g;
+	# Remove ExecStart options variables
+	# ExecStart=/usr/sbin/sshd -D $OPTIONS
+	# ExecStart=/usr/sbin/sshd -D $SSHD_OPTS
+	$ext_sshd_service_file_contents =~ s/^\s*(ExecStart=.+\S)\s+\$\S*OPT\S*(.*)$/$1$2/gm;
+	
+	# Remove explicit -f arguments from ExecStart line
+	$ext_sshd_service_file_contents =~ s/^\s*(ExecStart=.+\S)\s+-f\s+\S+(.*)$/$1$2/gm;
+	
+	# Add -f argument to ExecStart line
+	$ext_sshd_service_file_contents =~ s|^\s*(ExecStart=.+\S)\s*$|$1 -f /etc/ssh/external_sshd_config|gm;
 	
 	# Set EnvironmentFile to /dev/null, service won't start if the file doesn't exist
 	$ext_sshd_service_file_contents =~ s|(EnvironmentFile)=.*|$1=/dev/null|g;
 	
-	$ext_sshd_service_file_contents .= "\n";
+	# Remove Alias= line which may exist in ssh_config:
+	# Alias=ext_sshd.service
+	# Otherwise, this may occur when attempting to enable the service if the service is named the same as the alias:
+	# Failed to execute operation: Too many levels of symbolic links
+	$ext_sshd_service_file_contents =~ s/^\s*Alias=.*//gm;
+	
+	# Add explicit lines, remove first to avoid duplicates:
+	$ext_sshd_service_file_contents =~ s/^\s*(Restart|RestartSec|StartLimitInterval)=.*\n?//gm;
+	
+	# Attempt to restart if the service dies
+	$ext_sshd_service_file_contents =~ s/(\[Service\])/$1\nRestart=on-failure/gm;
+	$ext_sshd_service_file_contents =~ s/(\[Service\])/$1\nRestartSec=3s/gm;
+	
+	# (VCL-1027) Add StartLimitInterval=0 under [Service] to prevent:
+	#    Job for ext_sshd.service failed because start of the service was attempted too often
+	$ext_sshd_service_file_contents =~ s/(\[Service\])/$1\nStartLimitInterval=0/gm;
+	
+	notify($ERRORS{'DEBUG'}, 0, "$ext_sshd_service_file_path:\n$ext_sshd_service_file_contents");
 	
 	if (!$self->os->create_text_file($ext_sshd_service_file_path, $ext_sshd_service_file_contents)) {
 		notify($ERRORS{'WARNING'}, 0, "failed to create ext_sshd service file on $computer_node_name: $ext_sshd_service_file_path");
@@ -613,7 +614,7 @@ sub add_ext_sshd_service {
 	return $self->enable_service('ext_sshd');
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _daemon_reload
 
@@ -650,7 +651,65 @@ sub _daemon_reload {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
+
+=head2 _get_service_unit_file_path
+
+ Parameters  : $service_name
+ Returns     : string
+ Description : Determines the unit file for the service specified by the
+               argument. This is needed because the file name is not always
+               $service_name.service. This is the case when a service has alias
+               names configured such as the ssh and sshd services on Ubuntu 16.
+               The file path for the sshd service is ssh.service.
+
+=cut
+
+sub _get_service_unit_file_path {
+	my $self = shift;
+	if (ref($self) !~ /VCL::/i) {
+		notify($ERRORS{'CRITICAL'}, 0, "subroutine was called as a function, it must be called as a class method");
+		return 0;
+	}
+	
+	my $service_name = shift;
+	if (!$service_name) {
+		notify($ERRORS{'WARNING'}, 0, "service name argument was not supplied");
+		return;
+	}
+	
+	my $computer_node_name = $self->data->get_computer_node_name();
+	
+	my $command = "systemctl show $service_name.service --property=FragmentPath";
+	my ($exit_status, $output) = $self->os->execute($command, 0);
+	if (!defined($output)) {
+		notify($ERRORS{'WARNING'}, 0, "failed to retrieve unit file path for $service_name service on $computer_node_name: $command");
+		return;
+	}
+	elsif ($exit_status ne 0 || grep(/(failed)/i, @$output)) {
+		notify($ERRORS{'WARNING'}, 0, "failed to retrieve unit file path for $service_name service on $computer_node_name, exit status: $exit_status, output:\n" . join("\n", @$output));
+		return;
+	}
+	
+	# Expected output:
+	# FragmentPath=/lib/systemd/system/ssh.service
+	my ($file_path_line) = grep(/FragmentPath=/, @$output);
+	if (!$file_path_line) {
+		notify($ERRORS{'WARNING'}, 0, "failed to retrieve unit file path for $service_name service on $computer_node_name, output does not contain a 'FragmentPath=' line, output:\n" . join("\n", @$output));
+		return;
+	}
+	
+	my ($file_path) = $file_path_line =~ /FragmentPath=(.+)\s*$/g;
+	if (!$file_path) {
+		notify($ERRORS{'WARNING'}, 0, "failed to retrieve unit file path for $service_name service on $computer_node_name, failed to parse 'FragmentPath=' line, output:\n" . join("\n", @$output));
+		return;
+	}
+	
+	notify($ERRORS{'DEBUG'}, 0, "retrieved unit file path for $service_name service on $computer_node_name: $file_path");
+	return $file_path
+}
+
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
