@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: Provisioning.pm 1670472 2015-03-31 20:34:45Z jfthomps $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -32,7 +32,7 @@ VCL::Module::Provisioning - VCL provisioning base module
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::Module::Provisioning;
 
 # Specify the lib path using FindBin
@@ -43,7 +43,7 @@ use lib "$FindBin::Bin/../..";
 use base qw(VCL::Module);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -51,22 +51,22 @@ use 5.008000;
 use strict;
 use warnings;
 use diagnostics;
-use English qw( -no_match_vars );
+use English qw(-no_match_vars);
 use File::Basename;
 
 use VCL::utils;
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 node_status
 
- Parameters  : $computer_id (optional)
+ Parameters  : none
  Returns     : string
  Description : Checks the status of the computer in order to determine if the
                computer is ready to be reserved or needs to be reloaded. A
@@ -76,6 +76,7 @@ use VCL::utils;
                   * It is accessible
                   * It is loaded with the correct image
                   * OS module's post-load tasks have run
+                  * Computer has not been tagged with a tainted tag
                'POST_LOAD':
                   * Computer is loaded with the correct image
                   * OS module's post-load tasks have not run
@@ -86,138 +87,66 @@ use VCL::utils;
 =cut
 
 sub node_status {
-	my $self;
-	
-	# Get the argument
-	my $argument = shift;
-	
-	# Check if this subroutine was called an an object method or an argument was passed
-	if (ref($argument) =~ /VCL::Module/i) {
-		$self = $argument;
-	}
-	elsif (!ref($argument) || ref($argument) eq 'HASH') {
-		# An argument was passed, check its type and determine the computer ID
-		my $computer_id;
-		if (ref($argument)) {
-			# Hash reference was passed
-			$computer_id = $argument->{id};
-		}
-		elsif ($argument =~ /^\d+$/) {
-			# Computer ID was passed
-			$computer_id = $argument;
-		}
-		else {
-			# Computer name was passed
-			($computer_id) = get_computer_ids($argument);
-		}
-		
-		if ($computer_id) {
-			notify($ERRORS{'DEBUG'}, 0, "computer ID: $computer_id");
-		}
-		else {
-			notify($ERRORS{'DEBUG'}, 0, "unable to determine computer ID from argument:\n" . format_data($argument));
-			return;
-		}
-		
-		# Create a DataStructure object containing data for the computer specified as the argument
-		my $data;
-		eval {
-			$data= new VCL::DataStructure({computer_identifier => $computer_id});
-		};
-		if ($EVAL_ERROR) {
-			notify($ERRORS{'WARNING'}, 0, "failed to create DataStructure object for computer ID: $computer_id, error: $EVAL_ERROR");
-			return;
-		}
-		elsif (!$data) {
-			notify($ERRORS{'WARNING'}, 0, "failed to create DataStructure object for computer ID: $computer_id, DataStructure object is not defined");
-			return;
-		}
-		else {
-			notify($ERRORS{'DEBUG'}, 0, "created DataStructure object  for computer ID: $computer_id");
-		}
-		
-		# Create a provisioning object
-		my $provisioning_module_perl_package = $data->get_computer_provisioning_module_perl_package();
-		if ($self = ($provisioning_module_perl_package)->new({data_structure => $data})) {
-			notify($ERRORS{'DEBUG'}, 0, "created $provisioning_module_perl_package object to check the status of computer ID: $computer_id");
-		}
-		else {
-			notify($ERRORS{'WARNING'}, 0, "failed to create $provisioning_module_perl_package object to check the status of computer ID: $computer_id");
-			return;
-		}
-		
-		# Create an OS object for the provisioning object to access
-		if (!$self->create_os_object()) {
-			notify($ERRORS{'WARNING'}, 0, "failed to create OS object");
-			return;
-		}
+	my $self = shift;
+	unless (ref($self) && $self->isa('VCL::Module')) {
+		notify($ERRORS{'CRITICAL'}, 0, "subroutine was called as a function, it must be called as a class method");
+		return;
 	}
 	
-	my $reservation_id = $self->data->get_reservation_id();
 	my $computer_name = $self->data->get_computer_node_name();
 	my $image_name = $self->data->get_image_name();
-	my $request_forimaging = $self->data->get_request_forimaging();
-	my $imagerevision_id = $self->data->get_imagerevision_id();
+	my $reservation_imagerevision_id = $self->data->get_imagerevision_id();
 	
-	notify($ERRORS{'DEBUG'}, 0, "attempting to check the status of computer $computer_name, image: $image_name imagerevision_id; $imagerevision_id");
-	
-	# Create a hash reference and populate it with the default values
-	my $status;
-	$status->{currentimage} = '';
-	$status->{ssh} = 0;
-	$status->{image_match} = 0;
-	$status->{status} = 'RELOAD';
+	notify($ERRORS{'DEBUG'}, 0, "checking if $computer_name is responding and loaded with $image_name, imagerevision ID: $reservation_imagerevision_id");
 	
 	# Check if SSH is available
-	if ($self->os->is_ssh_responding()) {
-		notify($ERRORS{'DEBUG'}, 0, "$computer_name is responding to SSH");
-		$status->{ssh} = 1;
-	}
-	else {
+	if (!$self->os->is_ssh_responding()) {
 		notify($ERRORS{'OK'}, 0, "$computer_name is not responding to SSH, returning 'RELOAD'");
-		$status->{status} = 'RELOAD';
-		$status->{ssh} = 0;
-		
-		# Skip remaining checks if SSH isn't available
-		return $status;
+		return 'RELOAD';
 	}
 	
-	# Get the contents of currentimage.txt and check if currentimage.txt matches the requested image name
-	my $current_image_revision_id = $self->os->get_current_image_info();
-	$status->{currentimagerevision_id} = $current_image_revision_id;
-
-	$status->{currentimage} = $self->data->get_computer_currentimage_name();
-	my $vcld_post_load_status = $self->data->get_computer_currentimage_vcld_post_load(0);
-	
+	# Check if the imagerevision ID loaded on the computer matches the reservation
+	my $current_image_revision_id = $self->os->get_current_imagerevision_id();
 	if (!$current_image_revision_id) {
-		notify($ERRORS{'OK'}, 0, "unable to retrieve currentimage.txt contents on $computer_name, returning 'RELOAD'");
-		return $status;
+		notify($ERRORS{'OK'}, 0, "unable to retrieve imagerevision ID from $computer_name, returning 'RELOAD'");
+		return 'RELOAD';
 	}
-	#elsif ($current_image_name eq $image_name) {
-	elsif ($current_image_revision_id eq $imagerevision_id) {
-		notify($ERRORS{'DEBUG'}, 0, "currentimage.txt image $current_image_revision_id ($status->{currentimage}) matches requested image $imagerevision_id ($image_name) on $computer_name");
-		$status->{image_match} = 1;
+	elsif ($current_image_revision_id ne $reservation_imagerevision_id) {
+		notify($ERRORS{'OK'}, 0, "$computer_name is loaded with imagerevision ID: $current_image_revision_id, not $reservation_imagerevision_id, returning 'RELOAD'");
+		return 'RELOAD';
 	}
 	else {
-		notify($ERRORS{'OK'}, 0, "currentimage.txt image $current_image_revision_id ($status->{currentimage}) does not match requested image name $imagerevision_id ($image_name) on $computer_name, returning 'RELOAD'");
-		return $status;
+		notify($ERRORS{'DEBUG'}, 0, "$computer_name is loaded with the correct imagerevision ID: $current_image_revision_id");
 	}
 	
-	# Check if the OS post_load tasks have run
-	if ($vcld_post_load_status) {
-		notify($ERRORS{'DEBUG'}, 0, "OS module post_load tasks have been completed on $computer_name");
-		$status->{status} = 'READY';
-	}
-	else {
-		notify($ERRORS{'DEBUG'}, 0, "OS module post_load tasks have not been completed on $computer_name, returning 'POST_LOAD'");
-		$status->{status} = 'POST_LOAD';
+	# Check if current image has been tagged as tainted
+	my $tainted_status = $self->os->get_tainted_status();
+	if ($tainted_status) {
+		notify($ERRORS{'WARNING'}, 0, "user may have previously had the ability to log in to the image currently loaded on $computer_name, current image is tagged as tainted, returning 'RELOAD'");
+		return 'RELOAD';
 	}
 	
-	notify($ERRORS{'OK'}, 0, "status of $computer_name: $status->{status}");
-	return $status;
+	# Check if the post-load tasks have been completed
+	my $post_load_status = $self->os->get_post_load_status();
+	if (!$post_load_status) {
+		notify($ERRORS{'DEBUG'}, 0, "OS module post_load tasks have NOT been completed on $computer_name, returning 'POST_LOAD'");
+		return 'POST_LOAD';
+	}
+	
+	# Check if OS module implements a node_status_os_check subroutine
+	# Currently, this is only used by the Windows module to ensure the AD configuration is correct if an image's AD configuration is changed after a computer is loaded
+	if ($self->os->can('node_status_os_check')) {
+		if (!$self->os->node_status_os_check()) {
+			notify($ERRORS{'DEBUG'}, 0, "OS module's node_status_os_check returned false, returning 'RELOAD'");
+			return 'RELOAD';
+		}
+	}
+	
+	notify($ERRORS{'DEBUG'}, 0, "general node status checks all succeeded, returning 'READY'");
+	return 'READY';
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 get_image_repository_search_paths
 
@@ -255,7 +184,7 @@ sub get_image_repository_search_paths {
 	return @search_paths;
 } ## end sub get_image_repository_search_paths
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 retrieve_image
 
@@ -492,12 +421,28 @@ sub retrieve_image {
 	}
 	
 	# Copy each file path to the image repository directory
+	my $image_size_total_bytes = $partner_info{$retrieval_partner}{image_size};
+	my $image_size_total_gb = format_number(($image_size_total_bytes / 1024 / 1024 / 1024), 1);
+	
+	my $image_size_retrieved_bytes = 0;
+	my $image_size_retrieved_gb = 0.0;
+	
+	my $file_total_count = scalar(keys %{$partner_info{$retrieval_partner}{file_paths}});
+	my $file_retrieved_count = 0;
+	
 	for my $partner_file_path (sort {lc($a) cmp lc($b)} keys %{$partner_info{$retrieval_partner}{file_paths}}) {
+		$file_retrieved_count++;
 		my $file_name = $partner_info{$retrieval_partner}{file_paths}{$partner_file_path}{file_name};
 		my $local_file_path = "$image_repository_path_local/$file_name";
 		
+		my $file_size_bytes = $partner_info{$retrieval_partner}{file_paths}{$partner_file_path}{file_size};
+		
+		notify($ERRORS{'DEBUG'}, 0, "retrieving image file $file_retrieved_count/$file_total_count from $retrieval_partner_hostname: $partner_file_path --> $local_file_path (" . get_file_size_info_string($file_size_bytes) . ')');
 		if (run_scp_command("$partner_info{$retrieval_partner}{user}\@$retrieval_partner:$partner_file_path", $local_file_path, $partner_info{$retrieval_partner}{key}, $partner_info{$retrieval_partner}{port})) {
-			notify($ERRORS{'OK'}, 0, "image $image_name was copied from $retrieval_partner_hostname");
+			$image_size_retrieved_bytes += $file_size_bytes;
+			$image_size_retrieved_gb = format_number(($image_size_retrieved_bytes / 1024 / 1024 / 1024), 1);
+			my $retrieved_percent = format_number(int($image_size_retrieved_bytes / $image_size_total_bytes * 100), 0);			
+			notify($ERRORS{'OK'}, 0, "retrieved image file $file_retrieved_count/$file_total_count, $retrieved_percent\% complete, $image_size_retrieved_gb/$image_size_total_gb GB");
 		}
 		else {
 			notify($ERRORS{'WARNING'}, 0, "failed to copy image $image_name from $retrieval_partner_hostname");
@@ -514,7 +459,7 @@ sub retrieve_image {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_power_off
 
@@ -592,7 +537,7 @@ sub wait_for_power_off {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
