@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: xCAT.pm 1670472 2015-03-31 20:34:45Z jfthomps $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -39,7 +39,7 @@ VCL::Provisioning::xCAT - VCL module to support the xCAT provisioning engine
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::Module::Provisioning::xCAT;
 
 # Specify the lib path using FindBin
@@ -50,7 +50,7 @@ use lib "$FindBin::Bin/../../..";
 use base qw(VCL::Module::Provisioning);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -58,15 +58,16 @@ use 5.008000;
 use strict;
 use warnings;
 use diagnostics;
-use English qw( -no_match_vars );
+use English qw(-no_match_vars);
 
 use VCL::utils;
 use Fcntl qw(:DEFAULT :flock);
 use File::Copy;
 use IO::Seekable;
 use Socket;
+use version;
 
-##############################################################################
+###############################################################################
 
 =head1 CLASS ATTRIBUTES
 
@@ -84,13 +85,13 @@ use Socket;
 # Class attributes to store xCAT configuration details
 my $XCAT_ROOT;
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 initialize
 
@@ -142,7 +143,7 @@ sub initialize {
 	return 1;
 } ## end sub initialize
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 unload
 
@@ -167,7 +168,7 @@ sub unload {
 
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 load
 
@@ -340,7 +341,7 @@ sub load {
 			
 			if (my ($dhcpack_line) = grep(/DHCPACK/i, @dhcp_lines)) {
 				notify($ERRORS{'DEBUG'}, 0, "$computer_node_name acquired DHCP lease: '$dhcpack_line'");
-				if(!$dhcp_ack) {
+				if (!$dhcp_ack) {
 					insertloadlog($reservation_id, $computer_id, "xcatstage2", "acquired DHCP lease");
 					insertloadlog($reservation_id, $computer_id, "xcatround2", "waiting for boot flag");
 					$dhcp_ack=1;
@@ -452,7 +453,7 @@ sub load {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 capture
 
@@ -626,7 +627,7 @@ sub capture {
 	notify($ERRORS{'OK'}, 0, "successfully captured $image_name on $computer_node_name");
 	return 1;
 }
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 does_image_exist
 
@@ -687,22 +688,23 @@ sub does_image_exist {
 	else {
 		$du_command = "du -c $image_repository_path/*$image_name* 2>&1 | grep total 2>&1"
 	}
-	my ($du_exit_status, $du_output) = run_command($du_command);
+
+	my ($du_exit_status, $du_output) = $self->mn_os->execute($du_command);
 	
 	# If the partner doesn't have the image, a "no such file" error should be displayed
 	my $image_files_exist;
-	if (defined(@$du_output) && grep(/no such file/i, @$du_output)) {
+	if (!defined($du_output)) {
+		notify($ERRORS{'WARNING'}, 0, "failed to execute command $du_command");
+		return;
+	}
+	elsif (grep(/no such file/i, @$du_output)) {
 		notify($ERRORS{'OK'}, 0, "$image_name does NOT exist");
 		$image_files_exist = 0;
 	}
-	elsif (defined(@$du_output) && !grep(/\d+\s+total/i, @$du_output)) {
+	elsif (!grep(/\d+\s+total/i, @$du_output)) {
 		notify($ERRORS{'WARNING'}, 0, "du output does not contain a total line:\n" . join("\n", @$du_output));
 		return;
 	}
-	elsif (!defined($du_exit_status)) {
-		notify($ERRORS{'WARNING'}, 0, "failed to run ssh command to determine if image $image_name exists");
-		return;
-	}
 	
 	# Return 1 if the image size > 0
 	my ($image_size) = (@$du_output[0] =~ /(\d+)\s+total/);
@@ -782,7 +784,7 @@ sub does_image_exist {
 
 } ## end sub does_image_exist
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2  get_image_size
 
@@ -846,8 +848,209 @@ sub get_image_size {
 
 } ## end sub get_image_size
 
+#//////////////////////////////////////////////////////////////////////////////
 
-#/////////////////////////////////////////////////////////////////////////////
+=head2 get_nodetype_image_os_name
+
+ Parameters  : $image_name
+ Returns     : string
+ Description : Determines the name of the directory where installation files
+               should reside under the management node's install path.
+               Examples:
+					* image
+               * centos5
+					* rhels7.2
+					* ubuntu16.04.1
+               
+					The path is determined by first checking if a directory exists
+					matching the database values:
+					* managementnode.installpath (ex: /install)
+					* OS.sourcepath (ex: rhel7)
+					* image.architecture (ex: x86_64)
+					
+					Based on these values, the default path will be:
+					/install/rhel7/x86_64
+               
+               If a directory exactly matching OS.sourcepath cannot be located
+               on the managementnode node, an attempt is made to locate an
+               alternate suitable directory matching the distribution and major
+               version. Example, if OS.sourcepath = 'rhel7' and the default
+               directory does not exist:
+               /install/rhel7/x86_64
+               
+               Any of the following paths which exist on the management node may
+               be returned:
+               /install/rhel7.1/x86_64
+               /install/rhels7.2/x86_64
+               
+               If all of these paths exist, the path with the highest version is
+               returned:
+               rhels7.2
+               
+               Note: for 'rhel', both 'rhel' and 'rhels' are checked.
+
+=cut
+
+
+sub get_nodetype_image_os_name {
+	my $self = shift;
+	unless (ref($self) && $self->isa('VCL::Module')) {
+		notify($ERRORS{'CRITICAL'}, 0, "subroutine can only be called as a VCL::Module module object method");
+		return;	
+	}
+	
+	# Get the image name argument
+	my $image_name = shift || $self->data->get_image_name();
+	
+	# Check if path has already been determined
+	if (defined($self->{xcat_image_os_name}{$image_name})) {
+		return $self->{xcat_image_os_name}{$image_name};
+	}
+	
+	my $management_node_hostname = $self->data->get_management_node_hostname();
+	my $management_node_install_path = $self->data->get_management_node_install_path() || return;
+	
+	# Create a DataStructure object containing info about the image
+	my $image_data = $self->create_datastructure_object({image_identifier => $image_name}) || return;
+	my $os_install_type    = $image_data->get_image_os_install_type() || return;
+	my $os_source_path     = $image_data->get_image_os_source_path() || return;
+	my $image_architecture = $image_data->get_image_architecture() || return;
+	
+	if ($os_install_type =~ /image/i) {
+		notify($ERRORS{'DEBUG'}, 0, "OS install type for image $image_name is $os_install_type, returning 'image'");
+		$self->{xcat_image_os_name}{$image_name} = 'image';
+		return 'image';
+	}
+	elsif ($os_install_type !~ /(kickstart|netboot)/) {
+		notify($ERRORS{'WARNING'}, 0, "unable to determine nodetype image OS name for image $image_name, OS install type is not supported: $os_install_type");
+		return;
+	}
+	
+	# Remove trailing / from $management_node_install_path if exists
+	$management_node_install_path =~ s/\/+$//g;
+	
+	# Remove leading and trailing slashes from $os_source_path if exists
+	$os_source_path =~ s/^\/+//g;
+	$os_source_path =~ s/\/+$//g;
+
+	notify($ERRORS{'DEBUG'}, 0, "attempting to determine nodetype OS name for image on $management_node_hostname:\n" .
+		"image name      : $image_name\n" .
+		"OS install type : $os_install_type\n" .
+		"install path    : $management_node_install_path\n" .
+		"OS source path  : $os_source_path\n" .
+		"architecture    : $image_architecture"
+	);
+	
+	my $installation_repository_directory_path = "$management_node_install_path/$os_source_path/$image_architecture";
+	
+	# Check if the default path exists - it's often named something different
+	# xCAT's copycds command will use something like /install/rhels6.6
+	# OS.sourcepath is probably set to rhel6
+	# Creating a symlink doesn't work correctly because xCAT fails to parse directory names which don't contain a period correctly
+	if ($self->mn_os->file_exists($installation_repository_directory_path)) {
+		$self->{xcat_image_os_name}{$image_name} = $os_source_path;
+		notify($ERRORS{'DEBUG'}, 0, "default installation repository directory exists: $installation_repository_directory_path, returning '$self->{xcat_image_os_name}{$image_name}'");
+		return $self->{xcat_image_os_name}{$image_name};
+	}
+	
+	# Parse the version of the requested OS source path
+	my ($os_distribution_name, $os_version_string, $major_os_version_string) = $os_source_path =~ /^([a-z]+)((\d+)[\d\.]*)$/ig;
+	if (!defined($os_distribution_name) || !defined($os_version_string) || !defined($major_os_version_string)) {
+		$self->{xcat_image_os_name}{$image_name} = $os_source_path;
+		notify($ERRORS{'WARNING'}, 0, "failed to determine nodetype OS name for image $image_name, OS.sourcepath could not be parsed: $os_source_path, returning default path: '$self->{xcat_image_os_name}{$image_name}'");
+		return $self->{xcat_image_os_name}{$image_name};
+	}
+	
+	notify($ERRORS{'DEBUG'}, 0, "default installation repository directory path does not exist: $installation_repository_directory_path, attempting to locate another suitable path matching distribution: $os_distribution_name, version: $os_version_string, major version: $major_os_version_string");
+	
+	# Fix regex for 'rhel' and 'rhels'
+	my $os_distribution_regex = $os_distribution_name;
+	if ($os_distribution_name =~ /rhel/) {
+		$os_distribution_regex = 'rhels?';
+	}
+	
+	my $highest_version_string;
+	my $highest_version_directory_path;
+	my $highest_version_nodetype_os_name;
+	
+	# Retrieve list of directories under the root management node install path
+	my @check_directory_paths = $self->mn_os->find_files($management_node_install_path, "*", 0, 'd');
+	for my $check_directory_path (@check_directory_paths) {
+		# Remove trailing slash
+		$check_directory_path =~ s/\/+$//g;
+		
+		next if $check_directory_path eq $management_node_install_path;
+		
+		# Ignore directories that don't contain the Linux OS distribution name
+		if ($check_directory_path !~ /$os_distribution_regex/) {
+			#notify($ERRORS{'DEBUG'}, 0, "ignoring directory: $check_directory_path, it does not match the pattern for the OS distribution: '$os_distribution_regex'");
+			next;
+		}
+		
+		my ($check_nodetype_os_name) = $check_directory_path =~ /\/([^\/]+)$/;
+		if (!defined($check_nodetype_os_name)) {
+			notify($ERRORS{'WARNING'}, 0, "ignoring directory: $check_directory_path, failed to parse directory name (nodetype OS name)");
+			next;
+		}
+		
+		# Parse the version and major version from the directory name
+		my ($directory_version_string, $directory_major_version_string) = $check_directory_path =~ /$os_distribution_regex((\d+)[\d\.]*)/;
+		if (!defined($directory_version_string) || !defined($directory_major_version_string)) {
+			notify($ERRORS{'DEBUG'}, 0, "ignoring directory: $check_directory_path, version could not be determined");
+			next;
+		}
+		
+		# Make sure the major version matches
+		if ($directory_major_version_string ne $major_os_version_string) {
+			notify($ERRORS{'DEBUG'}, 0, "ignoring directory: $check_directory_path, major version $directory_major_version_string does not match requested major version $major_os_version_string");
+			next;
+		}
+		
+		# Make sure the correct architecture subdirectory exists
+		my $check_installation_repository_directory_path = "$check_directory_path/$image_architecture";
+		if (!$self->mn_os->file_exists($check_installation_repository_directory_path)) {
+			notify($ERRORS{'DEBUG'}, 0, "ignoring directory: $check_directory_path, '$image_architecture' subdirectory does not exist");
+			next;
+		}
+		
+		if (!$highest_version_string) {
+			notify($ERRORS{'DEBUG'}, 0, "1st matching directory is possibly an alternate path: $check_installation_repository_directory_path, version: $directory_version_string");
+			$highest_version_string = $directory_version_string;
+			$highest_version_directory_path = $check_installation_repository_directory_path;
+			$highest_version_nodetype_os_name = $check_nodetype_os_name;
+			next;
+		}
+		
+		# Check if the version isn't less than one previously checked
+		# Use version->declare->numify to correctly compare versions, otherwise 6.9 > 6.10
+		my $matching_version_numified = version->declare("$directory_version_string")->numify;
+		my $highest_matching_version_numified = version->declare("$highest_version_string")->numify;
+		if ($matching_version_numified <= $highest_matching_version_numified) {
+			notify($ERRORS{'DEBUG'}, 0, "directory ignored, version $directory_version_string ($matching_version_numified) is not higher than $highest_version_string ($highest_matching_version_numified): $check_directory_path");
+			next;
+		}
+		else {
+			notify($ERRORS{'DEBUG'}, 0, "directory version $directory_version_string ($matching_version_numified) is greater than $highest_version_string ($highest_matching_version_numified): $check_installation_repository_directory_path");
+			$highest_version_string = $directory_version_string;
+			$highest_version_directory_path = $check_installation_repository_directory_path;
+			$highest_version_nodetype_os_name = $check_nodetype_os_name;
+			next;
+		}
+	}
+	
+	if ($highest_version_nodetype_os_name) {
+		$self->{xcat_image_os_name}{$image_name} = $highest_version_nodetype_os_name;
+		notify($ERRORS{'OK'}, 0, "located alternate repository directory path on the local management node for kickstart image $image_name: $highest_version_directory_path, returning nodetype OS name: $self->{xcat_image_os_name}{$image_name}");
+		return $self->{xcat_image_os_name}{$image_name};
+	}
+	else {
+		$self->{xcat_image_os_name}{$image_name} = $os_source_path;
+		notify($ERRORS{'WARNING'}, 0, "failed to locate repository directory path on the local management node for kickstart image $image_name, returning default nodetype OS name: $self->{xcat_image_os_name}{$image_name}");
+		return $self->{xcat_image_os_name}{$image_name};
+	}
+}
+
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 get_image_repository_directory_path
 
@@ -869,69 +1072,72 @@ sub get_image_repository_directory_path {
 	
 	# Get the image name argument
 	my $image_name = shift || $self->data->get_image_name();
-	if (!$image_name) {
-		notify($ERRORS{'WARNING'}, 0, "image name argument was not specified");
-		return;
-	}
-
+	
 	# Check if a management node identifier argument was passed
 	my $management_node_identifier = shift;
+	my $management_node_hostname;
 	if ($management_node_identifier) {
-		notify($ERRORS{'DEBUG'}, 0, "management node identifier argument was specified: $management_node_identifier");
+		$management_node_hostname = $self->data->get_management_node_hostname($management_node_identifier);
+		if ($management_node_hostname) {
+			notify($ERRORS{'DEBUG'}, 0, "management node identifier argument was specified: $management_node_identifier, hostname: $management_node_hostname");
+		}
+		else {
+			notify($ERRORS{'WARNING'}, 0, "management node hostname could not be determined from argument: $management_node_identifier");
+			return;
+		}
+	}
+	else {
+		$management_node_hostname = $self->data->get_management_node_hostname();
 	}
 	
-	my $management_node_hostname = $self->data->get_management_node_hostname($management_node_identifier) || '';
-	return $self->{image_repository_path}{$image_name}{$management_node_hostname} if defined($self->{image_repository_path}{$image_name}{$management_node_hostname});
-	my $management_node_install_path = $self->data->get_management_node_install_path($management_node_identifier);
+	# Check if path has already been determined
+	if (defined($self->{xcat_image_repository_directory_path}{$image_name}{$management_node_hostname})) {
+		return $self->{xcat_image_repository_directory_path}{$image_name}{$management_node_hostname};
+	}
+	
+	my $management_node_install_path = $self->data->get_management_node_install_path($management_node_identifier) || return;
 	
 	# Create a DataStructure object containing info about the image
 	my $image_data = $self->create_datastructure_object({image_identifier => $image_name}) || return;
-	my $image_id = $image_data->get_image_id() || return;
-	my $image_os_name = $image_data->get_image_os_name() || return;
-	my $image_os_type = $image_data->get_image_os_type() || return;
-	my $image_os_install_type = $image_data->get_image_os_install_type() || return;
-	my $image_os_source_path = $image_data->get_image_os_source_path() || return;
+	my $os_install_type = $image_data->get_image_os_install_type() || return;
+	my $os_source_path = $image_data->get_image_os_source_path() || return;
 	my $image_architecture = $image_data->get_image_architecture() || return;
 	
-	# Remove trailing / from $image_os_source_path if exists
-	$image_os_source_path =~ s/\/$//;
+	# Remove trailing / from $management_node_install_path if exists
+	$management_node_install_path =~ s/\/+$//;
 	
-	#notify($ERRORS{'DEBUG'}, 0, "attempting to determine repository path for image on $management_node_hostname:
-	#	image id:        $image_id
-	#	OS name:         $image_os_name
-	#	OS type:         $image_os_type
-	#	OS install type: $image_os_install_type
-	#	OS source path:  $image_os_source_path\n
-	#	architecture:    $image_architecture
-	#");
+	# Remove trailing / from $os_source_path if exists
+	$os_source_path =~ s/\/+$//;
+
+	notify($ERRORS{'DEBUG'}, 0, "attempting to determine repository path for image on $management_node_hostname:\n" .
+		"install path    : $management_node_install_path\n" .
+		"image name      : $image_name\n" .
+		"OS install type : $os_install_type\n" .
+		"OS source path  : $os_source_path\n" .
+		"architecture    : $image_architecture"
+	);
 	
-	# If image OS source path has a leading /, assume it was meant to be absolute
-	# Otherwise, prepend the install path
-	my $image_install_path;
-	if ($image_os_source_path =~ /^\//) {
-		$image_install_path = $image_os_source_path;
+	
+	my $image_repository_directory_path;
+	if ($os_source_path =~ /^\//) {
+		# If image OS source path has a leading /, assume it was meant to be absolute
+		$image_repository_directory_path = $os_source_path;
+	}
+	elsif ($os_install_type eq 'kickstart') {
+		my $nodetype_image_os_name = $self->get_nodetype_image_os_name($image_name) || $os_source_path;
+		$image_repository_directory_path = "$management_node_install_path/$nodetype_image_os_name/$image_architecture";
 	}
 	else {
-		$image_install_path = "$management_node_install_path/$image_os_source_path";
-	}
-
-	# Note: $XCAT_ROOT has a leading /
-	# Note: $image_install_path has a leading /
-	if ($image_os_install_type eq 'kickstart') {
-		# Kickstart installs use the xCAT path for both repo and tmpl paths
-		my $kickstart_repo_path = "$image_install_path/$image_architecture";
-		$self->{image_repository_path}{$management_node_hostname} = $kickstart_repo_path;
-		notify($ERRORS{'DEBUG'}, 0, "kickstart install type, returning $kickstart_repo_path");
-		return $kickstart_repo_path;
+		# Partimage
+		$image_repository_directory_path = "$management_node_install_path/$os_source_path/$image_architecture";
 	}
 	
-	my $repo_path = "$image_install_path/$image_architecture";
-	$self->{image_repository_path}{$image_name}{$management_node_hostname} = $repo_path;
-	notify($ERRORS{'DEBUG'}, 0, "returning repository path for $management_node_hostname: $repo_path");
-	return $repo_path;
+	$self->{xcat_image_repository_directory_path}{$image_name}{$management_node_hostname} = $image_repository_directory_path;
+	notify($ERRORS{'DEBUG'}, 0, "determined repository directory path: $self->{xcat_image_repository_directory_path}{$image_name}{$management_node_hostname}");
+	return $self->{xcat_image_repository_directory_path}{$image_name}{$management_node_hostname};
 } ## end sub get_image_repository_directory_path
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 get_image_repository_search_paths
 
@@ -950,7 +1156,7 @@ sub get_image_repository_search_paths {
 	}
 	
 	my $management_node_identifier = shift || $self->data->get_management_node_hostname();
-	my $management_node_install_path = $self->data->get_management_node_install_path($management_node_identifier);
+	my $management_node_install_path = $self->data->get_management_node_install_path($management_node_identifier) || return;
 	my $image_name = $self->data->get_image_name();
 	my $image_architecture = $self->data->get_image_architecture();
 	
@@ -983,7 +1189,7 @@ sub get_image_repository_search_paths {
 	return @repository_search_paths;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 power_reset
 
@@ -1033,7 +1239,7 @@ sub power_reset {
 	return 1;
 } ## end sub power_reset
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 power_on
 
@@ -1079,7 +1285,7 @@ sub power_on {
 	return 1;
 } ## end sub power_on
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 power_off
 
@@ -1133,7 +1339,7 @@ sub power_off {
 	return 1;
 } ## end sub power_off
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 power_status
 
@@ -1174,7 +1380,7 @@ sub power_status {
 	}
 } ## end sub power_status
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _edit_nodelist
 
@@ -1213,7 +1419,6 @@ sub _edit_nodelist {
 	my $image_data = $self->create_datastructure_object({image_identifier => $image_name}) || return;
 	my $image_os_install_type = $image_data->get_image_os_install_type() || return;
 	my $image_project = $image_data->get_image_project() || return;
-	my $image_os_name = $image_data->get_image_os_name() || return;
 	
 	my $request_state_name = $self->data->get_request_state_name();
 	
@@ -1223,7 +1428,7 @@ sub _edit_nodelist {
 	# For HPC, use image project = vclhpc. There should be an xCAT postscript group named 'vclhpc' configured with specific HPC postscripts
 	
 	my $groups;
-	if ($request_state_name eq 'image') {
+	if ($request_state_name =~ /(image|checkpoint)/) {
 		# Image-based install or capture
 		$groups = "all,blade,image";
 	}
@@ -1256,7 +1461,7 @@ sub _edit_nodelist {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _edit_nodetype
 
@@ -1290,26 +1495,35 @@ sub _edit_nodetype {
 	
 	# Create a DataStructure object containing info about the image
 	my $image_data = $self->create_datastructure_object({image_identifier => $image_name}) || return;
-	my $image_architecture = $image_data->get_image_architecture();
-	my $image_os_install_type = $image_data->get_image_os_install_type();
-	my $image_os_name = $image_data->get_image_os_name();
+	
+	my $image_architecture		= $image_data->get_image_architecture();
+	my $image_os_install_type	= $image_data->get_image_os_install_type();
+	my $image_os_name				= $image_data->get_image_os_name();
 	
 	my $request_state_name = $self->data->get_request_state_name();
 	
-	if ($request_state_name eq 'image' || $image_os_install_type =~ /image/) {
-		$image_os_name = 'image';
+	my $nodetype_os;
+	if ($request_state_name =~ /(image|checkpoint)/ || $image_os_install_type =~ /image/) {
+		$nodetype_os = 'image';
+	}
+	elsif ($image_os_install_type =~ /kickstart/i) {
+		# Try to dynamically determine the value for nodetype.os
+		$nodetype_os = $self->get_nodetype_image_os_name($image_name);
+	}
+	else {
+		$nodetype_os = $image_os_name;
 	}
 	
-	my $command = "$XCAT_ROOT/bin/nodech $computer_node_name nodetype.os=$image_os_name nodetype.arch=$image_architecture nodetype.profile=$image_name";
+	my $command = "$XCAT_ROOT/bin/nodech $computer_node_name nodetype.os=$nodetype_os nodetype.arch=$image_architecture nodetype.profile=$image_name";
 	my ($exit_status, $output) = $self->mn_os->execute($command);
 	if (!defined($output)) {
-		notify($ERRORS{'WARNING'}, 0, "failed to execute command to edit xCAT configuration of $computer_node_name");
+		notify($ERRORS{'WARNING'}, 0, "failed to execute command to edit xCAT configuration of $computer_node_name: $command");
 		return;
 	}
 	elsif (grep(/Error/i, @$output)) {
 		# If an error occurs the output will look like this:
 		# Error: Invalid nodes and/or groups in noderange: vclh3-00
-		notify($ERRORS{'WARNING'}, 0, "failed to edit xCAT configuration of $computer_node_name, output:\n" . join("\n", @$output));
+		notify($ERRORS{'WARNING'}, 0, "failed to edit xCAT configuration of $computer_node_name, command: '$command'\noutput:\n" . join("\n", @$output));
 		return;
 	}
 	elsif (grep(/\w/, @$output)) {
@@ -1323,7 +1537,7 @@ sub _edit_nodetype {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _lsdef
 
@@ -1382,7 +1596,7 @@ sub _lsdef {
 	return $node_info;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2  _nodestat
 
@@ -1428,7 +1642,7 @@ sub _nodestat {
 	return;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _nodeset
 
@@ -1500,7 +1714,7 @@ sub _nodeset {
 	return;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2  _get_nodeset_all_stat_info
 
@@ -1544,7 +1758,7 @@ sub _get_nodeset_all_stat_info {
 	return $nodeset_stat_info;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _wait_for_on
 
@@ -1579,7 +1793,7 @@ sub _wait_for_on {
 	);
 } ## end sub _wait_for_on
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _wait_for_off
 
@@ -1614,7 +1828,7 @@ sub _wait_for_off {
 	);
 } ## end sub _wait_for_off
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _rpower
 
@@ -1765,7 +1979,7 @@ sub _rpower {
 	return;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _rinv
 
@@ -1841,7 +2055,7 @@ sub _rinv {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _get_tmpl_directory_path
 
@@ -1872,7 +2086,6 @@ sub _get_tmpl_directory_path {
 	if ($management_node_identifier) {
 		notify($ERRORS{'DEBUG'}, 0, "management node identifier argument was specified: $management_node_identifier");
 	}
-	my $management_node_install_path = $self->data->get_management_node_install_path($management_node_identifier);
 	
 	# Create a DataStructure object containing info about the image
 	my $image_data = $self->create_datastructure_object({image_identifier => $image_name}) || return;
@@ -1908,7 +2121,7 @@ sub _get_tmpl_directory_path {
 	return $image_template_path;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _create_template
 
@@ -1992,7 +2205,7 @@ sub _create_template {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _delete_template
 
@@ -2058,7 +2271,7 @@ sub _delete_template {
 	return 1;
 } ## end sub _delete_template
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _is_throttle_limit_reached
 
@@ -2167,7 +2380,7 @@ sub _is_throttle_limit_reached {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 _get_install_status
 
@@ -2227,7 +2440,7 @@ sub _get_install_status {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 check_image_os
 
@@ -2317,7 +2530,7 @@ EOF
 	return 1;
 } ## end sub check_image_os
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 DESTROY
 
@@ -2351,7 +2564,7 @@ sub DESTROY {
 		if (!defined($node) || !defined($request_state_name)) {
 			notify($ERRORS{'DEBUG'}, 0, "skipping xCAT DESTROY tasks, unable to retrieve node name and request state name from DataStructure");
 		}
-		elsif ($request_state_name =~ /^(new|reload|image)$/) {
+		elsif ($request_state_name =~ /^(new|reload|image|checkpoint)$/) {
 			notify($ERRORS{'DEBUG'}, 0, "request state is '$request_state_name', attempting to set nodeset state of $node to 'boot'");
 			$self->_nodeset($node, 'boot');
 		}
@@ -2364,11 +2577,11 @@ sub DESTROY {
 	$self->SUPER::DESTROY if $self->can("SUPER::DESTROY");
 } ## end sub DESTROY
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 initialize() if (!$XCAT_ROOT);
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
