@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: State.pm 1670477 2015-03-31 20:48:08Z arkurth $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -33,7 +33,7 @@ VCL::Core::State - VCL state base module
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::Module::State;
 
 # Specify the lib path using FindBin
@@ -44,7 +44,7 @@ use lib "$FindBin::Bin/../..";
 use base qw(VCL::Module);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -58,13 +58,13 @@ use POSIX qw(floor);
 use VCL::utils;
 use VCL::DataStructure;
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 initialize
 
@@ -157,13 +157,10 @@ sub initialize {
 		notify($ERRORS{'DEBUG'}, 0, "child reservation, not updating request state to 'pending'");
 	}
 	
-	# Set the PARENTIMAGE and SUBIMAGE keys in the request data hash
-	# These are deprecated, DataStructure's is_parent_reservation function should be used
-	$self->data->get_request_data->{PARENTIMAGE} = ($self->data->is_parent_reservation() + 0);
-	$self->data->get_request_data->{SUBIMAGE}    = (!$self->data->is_parent_reservation() + 0);
-	
-	# Set the parent PID and this process's PID in the hash
-	set_hash_process_id($self->data->get_request_data);
+	# Set the PID and PPID in the DataStructure
+	# These will be wrong if set in get_request_info before the state process is forked
+	$self->data->set_process_pid($PID);
+	$self->data->set_process_ppid(getppid() || '<unknown>');
 	
 	# Create an OS object
 	if (my $os = $self->create_os_object()) {
@@ -173,6 +170,13 @@ sub initialize {
 		$self->reservation_failed("failed to create OS object");
 	}
 	
+	# Set the os under mn_os to the OS object for the computer being loaded
+	# This allows the $self->mn_os object to call $self->os to retrieve the OS object for the computer being loaded
+	# This is useful because the DataStructure object changes when mn_os is created and it would otherwise not have access to the original data
+	if ($self->mn_os()) {
+		$self->mn_os->set_os($self->os);
+	}
+	
 	# Create a VM host OS object if vmhostid is set for the computer
 	my $vmhost_os;
 	if ($is_vm) {
@@ -229,7 +233,7 @@ sub initialize {
 	return 1;
 } ## end sub initialize
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 user_connected
 
@@ -262,11 +266,11 @@ sub user_connected {
 	my $ignore_connections_gte_min   = $self->os->get_timings('ignore_connections_gte');
 	my $ignore_connections_gte       = floor($ignore_connections_gte_min / 60);
 	
-	# Check if user deleted the request
-	$self->state_exit() if is_request_deleted($request_id);
-	
-	# Check if this is an imaging request, causes process to exit if state or laststate = image
-	$self->check_imaging_request();
+	# Check if the request state changed for any reason
+	# This will occur if the user deletes the request, makeproduction is initiated, reboot is initiated, image capture is started
+	if ($self->request_state_changed()) {
+		$self->state_exit();
+	}
 	
 	# Check if this is a server request, causes process to exit if server request
 	if ($server_request_id) {
@@ -308,29 +312,49 @@ sub user_connected {
 	return 0;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
-=head2 check_imaging_request
+=head2 request_state_changed
 
  Parameters  : none
  Returns     : boolean
- Description : The inuse process exits if the request state or laststate are set
-               to image, or if the forimaging flag has been set.
+ Description : Returns true if the neither current request state changed after
+               the process began, including:
+               * Request deleted
+               * Request deleted and makeproduction initiated
+               * Image capture initiated
+               * Checkpoint capture initiated
+               * Reboot initiated
 
 =cut
 
-sub check_imaging_request {
+sub request_state_changed {
 	my $self = shift;
-	my $request_id = $self->data->get_request_id();
+	if (ref($self) !~ /VCL::/) {
+		notify($ERRORS{'CRITICAL'}, 0, "subroutine was called as a function, it must be called as a class method, reservation failure tasks not attempted, process exiting");
+		exit 1;
+	}
 	
-	my $imaging_result = is_request_imaging($request_id);
-	if ($imaging_result eq 'image') {
-		notify($ERRORS{'OK'}, 0, "image creation process has begun, exiting");
-		$self->state_exit();
+	my $request_id = $self->data->get_request_id();
+	my $processing_request_state_name = $self->data->get_request_state_name();
+	
+	my ($current_state_name, $current_laststate_name) = get_request_current_state_name($request_id);
+	if (!$current_state_name || !$current_laststate_name) {
+		notify($ERRORS{'WARNING'}, 0, "request $request_id state data could not be retrieved, assuming request is deleted and was removed from the database, returning true");
+		return 1;
+	}
+	elsif (($current_state_name ne 'pending' and $current_state_name ne $processing_request_state_name) ||
+		($current_state_name eq 'pending' and $current_laststate_name ne $processing_request_state_name)) {
+		notify($ERRORS{'OK'}, 0, "request state changed after this process began: $processing_request_state_name --> $current_state_name/$current_laststate_name, returning true");
+		return 1;
+	}
+	else {
+		#notify($ERRORS{'DEBUG'}, 0, "request state has NOT changed after this process began: $processing_request_state_name --> $current_state_name/$current_laststate_name, returning false");
+		return 0;
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 reservation_failed
 
@@ -372,7 +396,6 @@ sub reservation_failed {
 
 	# Get the required data
 	my $request_id                  = $self->data->get_request_id();
-	my $request_logid               = $self->data->get_request_log_id();
 	my $reservation_id              = $self->data->get_reservation_id();
 	my $computer_id                 = $self->data->get_computer_id();
 	my $computer_short_name         = $self->data->get_computer_short_name();
@@ -415,7 +438,7 @@ sub reservation_failed {
 	my $new_computer_state_name;
 	my $request_log_ending;
 	
-	if ($request_state_name eq 'inuse') {
+	if ($request_state_name =~ /(inuse|reboot|server)/) {
 		# Check if the request end time has not been reached
 		my $request_end_time_epoch = convert_to_epoch_seconds($self->data->get_request_end_time());
 		my $current_time_epoch = time;
@@ -437,7 +460,7 @@ sub reservation_failed {
 		# Display the message
 		notify($ERRORS{'CRITICAL'}, 0, "reservation failed on $computer_short_name" . ($initialize_failed ? ', process failed to initialize' : '') . ": $message");
 		
-		if ($request_state_name eq 'image') {
+		if ($request_state_name =~ /(image|checkpoint)/) {
 			$new_request_state_name = 'maintenance';
 			$new_computer_state_name = 'maintenance';
 		}
@@ -485,7 +508,7 @@ sub reservation_failed {
 	$self->state_exit($new_request_state_name, $new_computer_state_name, $request_log_ending);
 } ## end sub reservation_failed
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 does_loadstate_exist_all_reservations
 
@@ -562,7 +585,7 @@ sub does_loadstate_exist_all_reservations {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 does_loadstate_exist_any_reservation
 
@@ -632,7 +655,7 @@ sub does_loadstate_exist_any_reservation {
 	return (wantarray) ? @exists : scalar(@exists);
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_all_reservations_to_begin
 
@@ -710,7 +733,7 @@ sub wait_for_all_reservations_to_begin {
 	return $result;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_reservation_loadstate
 
@@ -765,7 +788,7 @@ sub wait_for_reservation_loadstate {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_child_reservations_to_exit
 
@@ -834,7 +857,7 @@ sub wait_for_child_reservations_to_exit {
 	);
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 state_exit
 
@@ -856,17 +879,17 @@ sub state_exit {
 	$ENV{state_exit} = 1;
 	
 	my ($request_state_name_new, $computer_state_name_new, $request_log_ending) = @_;
+	notify($ERRORS{'DEBUG'}, 0, "beginning state module exit tasks, " .
+		"request state argument: " . ($request_state_name_new ? $request_state_name_new : '<not specified>') . ', ' .
+		"computer state argument: " . ($computer_state_name_new ? $computer_state_name_new : '<not specified>') . ', ' .
+		"log ending argument: " . ($request_log_ending ? $request_log_ending : '<not specified>')
+	);
 	
-	my $string = "beginning state module exit tasks\n";
-	$string .= "request state argument: " . ($request_state_name_new ? $request_state_name_new : '<not specified>') . "\n";
-	$string .= "computer state argument: " . ($computer_state_name_new ? $computer_state_name_new : '<not specified>') . "\n";
-	$string .= "log ending argument: " . ($request_log_ending ? $request_log_ending : '<not specified>');
-	notify($ERRORS{'DEBUG'}, 0, $string);
 	
 	my $calling_sub = get_calling_subroutine();
 	
 	my $request_id                 = $self->data->get_request_id();
-	my $request_logid              = $self->data->get_request_log_id();
+	my $request_logid              = $self->data->get_request_log_id(0);
 	my $reservation_id             = $self->data->get_reservation_id();
 	my @reservation_ids            = $self->data->get_reservation_ids();
 	my $reservation_count          = $self->data->get_reservation_count();
@@ -875,6 +898,7 @@ sub state_exit {
 	my $request_laststate_name_old = $self->data->get_request_laststate_name();
 	my $computer_id                = $self->data->get_computer_id();
 	my $computer_shortname         = $self->data->get_computer_short_name();
+	my $nathost_hostname           = $self->data->get_nathost_hostname(0);
 
 	if ($is_parent_reservation) {
 		# If parent of a cluster request, wait for child processes to exit before switching the state
@@ -921,66 +945,40 @@ sub state_exit {
 		
 		if ($request_state_name_new) {
 			# Never set request state to failed if previous state is image
-			if ($request_state_name_old eq 'image' && $request_state_name_new !~ /(complete|maintenance)/) {
+			# Allow pending/checkpoint --> reserved/checkpoint
+			if ($request_state_name_old =~ /(image|checkpoint)/ && $request_state_name_new !~ /(reserved|complete|maintenance)/) {
 				notify($ERRORS{'CRITICAL'}, 0, "previous request state is $request_state_name_old, not setting request state to $request_state_name_new, setting request and computer state to maintenance");
 				$request_state_name_new = 'maintenance';
 				$computer_state_name_new = 'maintenance';
 			}
-			elsif ($request_state_name_old eq 'inuse' && $request_state_name_new !~ /(inuse|timeout|maintenance)/) {
+			elsif ($request_state_name_old =~ /(inuse|reboot|server)/ && $request_state_name_new !~ /(inuse|timeout|maintenance)/) {
 				notify($ERRORS{'CRITICAL'}, 0, "previous request state is $request_state_name_old, not setting request state to $request_state_name_new, setting request and computer state to inuse");
 				$request_state_name_new = 'inuse';
 				$computer_state_name_new = 'inuse';
 			}
-			
-			# Update the reservation.lastcheck time to now if the next request state is inuse
-			# Do this to ensure that reservations are not processed again quickly after this process exits
-			# For cluster requests, the parent may have had to wait a while for child processes to exit
-			# Resetting reservation.lastcheck causes reservations to wait the full interval between inuse checks
-			if ($request_state_name_new =~ /(reserved|inuse)/) {
-				update_reservation_lastcheck(@reservation_ids);
-			}
-			
-			# Update the request state
-			if ($request_state_name_old ne 'deleted') {
-				if (is_request_deleted($request_id)) {
-					notify($ERRORS{'OK'}, 0, "request has been deleted, request state not updated: $request_state_name_old --> $request_state_name_new");
-				}
-				else {
-					# Check if the request state has already been updated
-					# This can occur if another reservation in a cluster failed
-					my ($request_state_name_current, $request_laststate_name_current) = get_request_current_state_name($request_id);
-					if ($request_state_name_current eq $request_state_name_new && $request_laststate_name_current eq $request_state_name_old) {
-						notify($ERRORS{'OK'}, 0, "request has NOT been deleted, current state already set to: $request_state_name_current/$request_laststate_name_current");
-					}
-					else {
-						notify($ERRORS{'OK'}, 0, "request has NOT been deleted, updating request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
-						if (!update_request_state($request_id, $request_state_name_new, $request_state_name_old)) {
-							notify($ERRORS{'WARNING'}, 0, "failed to change request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
-						}
-					}
-				}
-			}
-			else {
-				# Current request state = 'deleted'
-				if ($request_state_name_new =~ /(complete)/) {
-					if (!update_request_state($request_id, $request_state_name_new, $request_state_name_old)) {
-						notify($ERRORS{'WARNING'}, 0, "failed to change request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
-					}
-				}
-				else {
-					notify($ERRORS{'WARNING'}, 0, "request state not updated: $request_state_name_old --> $request_state_name_new");
-				}
-			}
-		}
-		
-		# Update log.ending if this is the parent reservation and argument was supplied
-		if ($request_log_ending) {
-			if (!update_log_ending($request_logid, $request_log_ending)) {
-				notify($ERRORS{'CRITICAL'}, 0, "failed to set log ending to $request_log_ending, log ID: $request_logid");
-			}
 		}
 	}
 	
+	
+	# If $request_log_ending was passed this should be the end of the reservation
+	# If NAT is used, rules added to the NAT host should be removed
+	if ($nathost_hostname) {
+		my $nat_sanitize_needed = 0;
+		if ($request_log_ending) {
+			notify($ERRORS{'DEBUG'}, 0, "attempting to sanitize firewall rules created for reservation $reservation_id on NAT host $nathost_hostname, \$request_log_ending argument was specified");
+			$nat_sanitize_needed = 1;
+		}
+		elsif ($request_state_name_new && $request_state_name_new =~ /(timeout|deleted|complete|image|checkpoint|failed)/) {
+			notify($ERRORS{'DEBUG'}, 0, "attempting to sanitize firewall rules created for reservation $reservation_id on NAT host $nathost_hostname, next request state is '$request_state_name_new'");
+			$nat_sanitize_needed = 1;
+		}
+		
+		if ($nat_sanitize_needed) {
+			$self->nathost_os->firewall->nat_sanitize_reservation();
+		}
+	}
+	
+	
 	# Update the computer state if argument was supplied
 	if ($computer_state_name_new) {
 		my $computer_state_name_old = $self->data->get_computer_state_name();
@@ -993,8 +991,9 @@ sub state_exit {
 		}
 	}
 	
-	# Clean computerloadlog as late as possible
+	
 	if ($is_parent_reservation) {
+		# Clean computerloadlog as late as possible
 		if ($request_state_name_old =~ /(new|reserved)/) {
 			# Only delete computerloadlog entries with loadstatename = 'begin' for all reservations in this request
 			delete_computerloadlog_reservation(\@reservation_ids, '(begin)');
@@ -1003,12 +1002,60 @@ sub state_exit {
 			# Delete all computerloadlog entries for all reservations in this request
 			delete_computerloadlog_reservation(\@reservation_ids);
 		}
+		
+		# Update log.ending if this is the parent reservation and argument was supplied
+		if ($request_logid && $request_log_ending) {
+			if (!update_log_ending($request_logid, $request_log_ending)) {
+				notify($ERRORS{'CRITICAL'}, 0, "failed to set log ending to $request_log_ending, log ID: $request_logid");
+			}
+		}
+		
+		# Update the reservation.lastcheck time to now if the next request state is inuse
+		# Do this to ensure that reservations are not processed again quickly after this process exits
+		# For cluster requests, the parent may have had to wait a while for child processes to exit
+		# Resetting reservation.lastcheck causes reservations to wait the full interval between inuse checks
+		if ($request_state_name_new && $request_state_name_new =~ /(reserved|inuse|reboot|server)/) {
+			update_reservation_lastcheck(@reservation_ids);
+		}
 	}
 	
 	# Insert a computerloadlog 'exited' entry
 	# This is used by the parent cluster reservation
+	# Do this as late as possible, if request.state is changed to 'complete', vcld may begin processing it before this process exits
+	# Warning will be generated if request is deleted before insertloadlog is executed
 	insertloadlog($reservation_id, $computer_id, "exited", "vcld process exiting");
 	
+	
+	if ($is_parent_reservation && $request_state_name_new) {	
+		# Update the request state
+		if ($request_state_name_old ne 'deleted') {
+			if (is_request_deleted($request_id)) {
+				notify($ERRORS{'OK'}, 0, "request has been deleted, request state not updated: $request_state_name_old --> $request_state_name_new");
+			}
+			else {
+				# Check if the request state has already been updated
+				# This can occur if another reservation in a cluster failed
+				my ($request_state_name_current, $request_laststate_name_current) = get_request_current_state_name($request_id);
+				if ($request_state_name_current eq $request_state_name_new && $request_laststate_name_current eq $request_state_name_old) {
+					notify($ERRORS{'OK'}, 0, "request has NOT been deleted, current state already set to: $request_state_name_current/$request_laststate_name_current");
+				}
+				else {
+					notify($ERRORS{'OK'}, 0, "request has NOT been deleted, updating request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
+					if (!update_request_state($request_id, $request_state_name_new, $request_state_name_old)) {
+						notify($ERRORS{'WARNING'}, 0, "failed to change request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
+					}
+				}
+			}
+		}
+		else {
+			# Current request state = 'deleted', always set the request state to 'complete'
+			if (!update_request_state($request_id, 'complete', $request_state_name_old)) {
+				notify($ERRORS{'WARNING'}, 0, "failed to change request state: $request_state_name_old/$request_laststate_name_old --> $request_state_name_new/$request_state_name_old");
+			}
+		}
+	}
+	
+	
 	# Don't call exit if this was called from DESTROY or else DESTROY gets called again
 	if ($calling_sub) {
 		if ($calling_sub =~ /DESTROY/) {
@@ -1025,7 +1072,7 @@ sub state_exit {
 	exit;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 DESTROY
 
@@ -1100,7 +1147,7 @@ sub DESTROY {
 	}
 } ## end sub DESTROY
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
