@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: image.pm 1670472 2015-03-31 20:34:45Z jfthomps $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -46,7 +46,7 @@ VCL::image - Perl module for the VCL image state
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::image;
 
 # Specify the lib path using FindBin
@@ -57,7 +57,7 @@ use lib "$FindBin::Bin/..";
 use base qw(VCL::Module::State);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -69,13 +69,13 @@ use English '-no_match_vars';
 
 use VCL::utils;
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 process
 
@@ -113,46 +113,27 @@ sub process {
 	my $computer_shortname         = $self->data->get_computer_short_name();
 	my $managementnode_shortname   = $self->data->get_management_node_short_name();
 	my $sysadmin_mail_address      = $self->data->get_management_node_sysadmin_email(0);
-
-	if ($sysadmin_mail_address) {
-		# Notify administrators that image creation is starting
-		my $body = <<"END";
-VCL Image Creation Started
-
-Request ID: $request_id
-Reservation ID: $reservation_id
-PID: $$
-
-Image ID: $image_id
-Image name: $image_name
-Base image size: $image_size
-Base revision ID: $imagerevision_id
-
-Management node: $managementnode_shortname
-
-Username: $user_unityid
-User ID: $user_id
-
-Computer ID: $computer_id
-Computer name: $computer_shortname
-
-Use Sysprep: $imagemeta_sysprep
-END
-		mail($sysadmin_mail_address, "VCL IMAGE Creation Started: $image_name", $body, $affiliation_helpaddress);
-	}
 	
-	# Make sure image does not exist in the repository
+	# Send an email to administrators indicating image capture started
+	if ($sysadmin_mail_address) {
+		my ($admin_subject, $admin_message) = $self->get_admin_message('image_creation_started');
+		if (defined($admin_subject) && defined($admin_message)) {
+			mail($sysadmin_mail_address, $admin_subject, $admin_message, $affiliation_helpaddress);
+		}
+	}
+
+	# Make sure image does not already exist
 	my $image_already_exists = $self->provisioner->does_image_exist();
 	if ($image_already_exists) {
-		notify($ERRORS{'CRITICAL'}, 0, "image $image_name already exists in the repository");
+		notify($ERRORS{'CRITICAL'}, 0, "image $image_name already exists");
 		$self->reservation_failed();
 	}
 	elsif (!defined($image_already_exists)) {
-		notify($ERRORS{'CRITICAL'}, 0, "image $image_name already partially exists in the repository");
+		notify($ERRORS{'CRITICAL'}, 0, "failed to determine if image $image_name already exists");
 		$self->reservation_failed();
 	}
 	else {
-		notify($ERRORS{'OK'}, 0, "image $image_name does not exist in the repository");
+		notify($ERRORS{'OK'}, 0, "image $image_name does not exist");
 	}
 	
 	# Get the current timestamp
@@ -173,6 +154,10 @@ END
 		$self->reservation_failed();
 	}
 	
+	# Make sure post_reservation scripts get executed before capturing computer
+	# This is normally done by reclaim.pm, but this won't be called in the following capture/reload sequence
+	$self->os->post_reservation();
+	
 	# Call the provisioning modules's capture() subroutine
 	# The provisioning module should do everything necessary to capture the image
 	notify($ERRORS{'OK'}, 0, "calling provisioning module's capture() subroutine");
@@ -191,9 +176,16 @@ END
 			$self->reservation_failed();
 		}
 		
-		if (!$self->os->post_load()) {
-			notify($ERRORS{'CRITICAL'}, 0, "failed to create checkpoint of image, unable to complete OS post-load tasks on $computer_shortname after image was captured and computer was powered on");
-			$self->reservation_failed();
+		# Check if the OS module implements a post_load subroutine
+		if ($self->os->can('post_load')) {
+			if ($self->os->post_load()) {
+				# Add a line to currentimage.txt indicating post_load has run
+				$self->os->set_post_load_status();
+			}
+			else {
+				notify($ERRORS{'CRITICAL'}, 0, "failed to create checkpoint of image, unable to complete OS post-load tasks on $computer_shortname after image was captured and computer was powered on");
+				$self->reservation_failed();
+			}
 		}
 		
 		if (!$self->os->reserve()) {
@@ -241,14 +233,18 @@ EOF
 		notify($ERRORS{'WARNING'}, 0, "image table could not be updated for image=$image_id");
 	}
 	
+	# Call the OS module's post_capture subroutine
+	# This call might be relocated to each provisioning module's process subroutine like the call to pre_capture
+	$self->os->post_capture();
+	
 	$self->reservation_successful($image_size);
 } ## end sub process
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 reservation_successful
 
- Parameters  : none
+ Parameters  : $image_size_old
  Returns     : exits
  Description : Handles final steps when an image capture is successful. Sends
                message to image creator. Inserts reload request into the
@@ -259,116 +255,51 @@ EOF
 
 sub reservation_successful {
 	my $self           = shift;
-	my $image_size_old = shift;
+	my $image_size_old = shift || 0;
 	
 	my $request_data               = $self->data->get_request_data();
-	my $request_id                 = $self->data->get_request_id();
 	my $request_state_name         = $self->data->get_request_state_name();
-	my $reservation_id             = $self->data->get_reservation_id();
-	my $user_id                    = $self->data->get_user_id();
-	my $user_unityid               = $self->data->get_user_login_id();
 	my $user_email                 = $self->data->get_user_email();
-	my $affiliation_sitewwwaddress = $self->data->get_user_affiliation_sitewwwaddress();
 	my $affiliation_helpaddress    = $self->data->get_user_affiliation_helpaddress();
-	my $image_id                   = $self->data->get_image_id();
-	my $image_name                 = $self->data->get_image_name();
-	my $image_prettyname           = $self->data->get_image_prettyname();
-	my $image_size                 = $self->data->get_image_size();
-	my $imagerevision_id           = $self->data->get_imagerevision_id();
-	my $imagemeta_sysprep          = $self->data->get_imagemeta_sysprep();
 	my $computer_id                = $self->data->get_computer_id();
-	my $computer_shortname         = $self->data->get_computer_short_name();
-	my $managementnode_shortname   = $self->data->get_management_node_short_name();
 	my $sysadmin_mail_address      = $self->data->get_management_node_sysadmin_email(0);
 	
-	# Send image creation successful email to user
-	my $subject_user;
-	my $body_user;
-	
-	if ($request_state_name eq 'checkpoint') {
-		$subject_user = "VCL -- $image_prettyname Image Checkpoint Succeeded";
-		$body_user = <<"END";
-
-Your VCL image checkpoint creation request for $image_prettyname has succeeded.
-
-You will need to visit the "Current Reservations" page and click "Connect" in order to be able to reconnect to the computer.
-
-Thank You,
-VCL Team
-END
+	# Send a capture completed message to the image owner
+	my ($user_subject, $user_message);
+	if ($request_state_name =~ /(checkpoint)/i) {
+		($user_subject, $user_message) = $self->get_user_message('image_checkpoint_success');
 	}
 	else {
-		$subject_user = "VCL -- $image_prettyname Image Creation Succeeded";
-		$body_user = <<"END";
-
-Your VCL image creation request for $image_prettyname has succeeded.
-
-Please visit $affiliation_sitewwwaddress and you should see an image called $image_prettyname.
-
-Please test this image to confirm it works correctly.
-
-Thank You,
-VCL Team
-END
+		($user_subject, $user_message) = $self->get_user_message('image_creation_success');
+	}
+	if (defined($user_subject) && defined($user_message)) {
+		mail($user_email, $user_subject, $user_message, $affiliation_helpaddress);
 	}
 	
-	mail($user_email, $subject_user, $body_user, $affiliation_helpaddress);
-	
-	
-	# Send mail to $sysadmin_mail_address
+	# Send mail to administrators
 	if ($sysadmin_mail_address) {
-		my $subject_admin;
-		if ($request_state_name eq 'checkpoint') {
-			$subject_admin = "VCL IMAGE Checkpoint Completed: $image_name"
+		# Get the administrator email subject and message
+		my ($admin_subject, $admin_message) = $self->get_admin_message('image_creation_complete');
+		if (defined($admin_subject) && defined($admin_message)) {
+			mail($sysadmin_mail_address, $admin_subject, $admin_message, $affiliation_helpaddress);
 		}
-		else {
-			$subject_admin = "VCL IMAGE Creation Completed: $image_name"
-		}
-		
-		my $body_admin = <<"END";
-VCL Image Creation Completed
-
-Request ID: $request_id
-Reservation ID: $reservation_id
-PID: $$
-
-Image ID: $image_id
-Image name: $image_name
-Image size change: $image_size_old --> $image_size
-
-Revision ID: $imagerevision_id
-
-Management node: $managementnode_shortname
-
-Username: $user_unityid
-User ID: $user_id
-
-Computer ID: $computer_id
-Computer name: $computer_shortname
-
-Use Sysprep: $imagemeta_sysprep
-END
-
-		mail($sysadmin_mail_address, $subject_admin, $body_admin, $affiliation_helpaddress);
 	}
 	
 	if ($request_state_name eq 'checkpoint') {
-		switch_state($request_data, 'reserved', 'checkpoint');
+		$self->state_exit('reserved', 'checkpoint');
 	}
 	else {
 		# Insert reload request data into the datbase
 		if (!insert_reload_request($request_data)) {
-			notify($ERRORS{'CRITICAL'}, 0, "failed to insert reload request into database for computer id=$computer_id");
+			notify($ERRORS{'CRITICAL'}, 0, "failed to insert reload request into database for computer ID: $computer_id");
 		}
 		
 		# Switch the request state to complete, leave the computer state as is, update log ending to EOR, exit
-		switch_state($request_data, 'complete', '', 'EOR', '1');
+		$self->state_exit('complete', undef, 'EOR');
 	}
-	
-	exit;
 } ## end sub reservation_successful
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 reservation_failed
 
@@ -384,135 +315,47 @@ sub reservation_failed {
 	my $self = shift;
 	
 	my $request_id                 = $self->data->get_request_id();
-	my $reservation_id             = $self->data->get_reservation_id();
-	my $user_id                    = $self->data->get_user_id();
-	my $user_unityid               = $self->data->get_user_login_id();
+	my $request_state_name         = $self->data->get_request_state_name();
+	my $request_laststate_name     = $self->data->get_request_laststate_name();
 	my $user_email                 = $self->data->get_user_email();
-	my $user_firstname             = $self->data->get_user_firstname() || '';
-	my $user_lastname              = $self->data->get_user_lastname() || '';
-	my $user_affiliation_name      = $self->data->get_user_affiliation_name();
 	my $affiliation_helpaddress    = $self->data->get_user_affiliation_helpaddress();
-	my $image_id                   = $self->data->get_image_id();
 	my $image_name                 = $self->data->get_image_name();
-	my $image_prettyname           = $self->data->get_image_prettyname();
-	my $imagerevision_id           = $self->data->get_imagerevision_id();
-	my $os_module_perl_package     = $self->data->get_image_os_module_perl_package();
-	my $imagemeta_sysprep          = $self->data->get_imagemeta_sysprep();
 	my $computer_id                = $self->data->get_computer_id();
 	my $computer_shortname         = $self->data->get_computer_short_name();
-	my $computer_state_name        = $self->data->get_computer_state_name();
-	my $provisioning_pretty_name   = $self->data->get_computer_provisioning_pretty_name();
-	my $provisioning_name          = $self->data->get_computer_provisioning_name();
-	my $provisioning_perl_package  = $self->data->get_computer_provisioning_module_perl_package();
-	my $managementnode_shortname   = $self->data->get_management_node_short_name();
 	my $sysadmin_mail_address      = $self->data->get_management_node_sysadmin_email(0);
-	my $vmhost_id                  = $self->data->get_vmhost_id() || '';
-	my $vmhost_computer_id         = $self->data->get_vmhost_computer_id() || '';
-	my $vmhost_short_name          = $self->data->get_vmhost_short_name() || '';
-	my $vmhost_profile_id          = $self->data->get_vmhost_profile_id() || '';
-	my $vmhost_profile_name        = $self->data->get_vmhost_profile_name() || '';
-	my $request_laststate_name     = $self->data->get_request_laststate_name();
-	
-	my $message = shift;
+	my $image_capture_type         = $self->data->get_image_capture_type();
 	
 	# Image process failed
+	my $message = shift;
 	if ($message) {
-		notify($ERRORS{'CRITICAL'}, 0, "$image_name image creation failed - $message");
+		notify($ERRORS{'CRITICAL'}, 0, "$image_name Image $image_capture_type Failed - $message");
 	}
 	else {
-		notify($ERRORS{'CRITICAL'}, 0, "$image_name image creation failed");
+		notify($ERRORS{'CRITICAL'}, 0, "$image_name Image $image_capture_type Failed");
 	}
 	
-	# Send mail to user
-	my $body_user = <<"END";
-
-We apologize for the inconvenience.
-Your image creation of $image_prettyname has been delayed
-due to a system issue that prevented the automatic completion.
-
-The image creation request and the computing resource have
-been placed in a safe mode. The VCL system administrators
-have been notified for manual intervention.
-
-Once the issues have been resolved, you will be notified
-by the successful completion email or contacted directly
-by the VCL system administrators.
-
-If you do not receive a response within one business day, please
-reply to this email.
-
-Thank You,
-VCL Team
-END
-	# Don't attempt to send another notice if $request_laststate_name is image. 
-	if ($request_laststate_name ne "maintenance") {
-		mail($user_email, "VCL -- NOTICE DELAY Image Creation $image_prettyname", $body_user, $affiliation_helpaddress);
+	# Send a capture delayed message to the image owner
+	my $user_message_key = 'image_creation_delayed';
+	my ($user_subject, $user_message) = $self->get_user_message($user_message_key);
+	if ($request_laststate_name ne "maintenance" && defined($user_subject) && defined($user_message)) {
+		mail($user_email, $user_subject, $user_message, $affiliation_helpaddress);
 	}
-
-	# Send mail to $sysadmin_mail_address
+	
+	# Send mail to administrators
 	if ($sysadmin_mail_address) {
-		my $body_admin = <<"END";
-VCL Image Creation Failed
-
-Management node: $managementnode_shortname
-
-Request ID: $request_id
-Reservation ID: $reservation_id
-PID: $$
-
-Image ID: $image_id
-Image revision ID: $imagerevision_id
-Image name: $image_name
-Image display name: $image_prettyname
-Image OS package: $os_module_perl_package
-
-User ID: $user_id
-User login name: $user_unityid
-User name: $user_firstname $user_lastname
-User affiliation: $user_affiliation_name
-
-Provisioning module: $provisioning_pretty_name ($provisioning_name)
-Provisioning package: $provisioning_perl_package
-
-Computer ID: $computer_id
-Computer name: $computer_shortname
-END
-		if ($vmhost_id) {
-			$body_admin .= <<"END";
-
-VM host ID: $vmhost_id
-VM host computer ID: $vmhost_computer_id
-VM host computer name: $vmhost_short_name
-
-VM host profile ID: $vmhost_profile_id
-VM host profile name: $vmhost_profile_name
-END
+		# Get the administrator email subject and message
+		# Pass a hash containing an IMAGE_CAPTURE_TYPE key - this gets replaced in the subject of the message
+		my $admin_message_key = 'image_creation_failed';
+		my ($admin_subject, $admin_message) = $self->get_admin_message($admin_message_key);
+		if (defined($admin_subject) && defined($admin_message)) {
+			mail($sysadmin_mail_address, $admin_subject, $admin_message, $affiliation_helpaddress);
 		}
-		notify($ERRORS{'OK'}, 0, "imaging reservation info:\n$body_admin");
-		mail($sysadmin_mail_address, "VCL -- NOTICE FAILED Image Creation $image_prettyname", $body_admin, $affiliation_helpaddress);
 	}
 	
-	# Update the request state to maintenance, laststate to image
-	if (update_request_state($request_id, "maintenance", "image")) {
-		notify($ERRORS{'OK'}, 0, "request state set to maintenance, laststate to image");
-	}
-	else {
-		notify($ERRORS{'CRITICAL'}, 0, "unable to set request state to maintenance, laststate to image");
-	}
-
-	# Update the computer state to maintenance
-	if (update_computer_state($computer_id, "maintenance")) {
-		notify($ERRORS{'OK'}, 0, "$computer_shortname state set to maintenance");
-	}
-	else {
-		notify($ERRORS{'CRITICAL'}, 0, "unable to set $computer_shortname state to maintenance");
-	}
-
-	notify($ERRORS{'OK'}, 0, "exiting");
-	exit;
+	$self->state_exit('maintenance', 'maintenance');
 } ## end sub reservation_failed
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 setup_get_menu
 
@@ -530,7 +373,7 @@ sub setup_get_menu {
 	};
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 setup_capture_base_image
 
@@ -725,7 +568,7 @@ sub setup_capture_base_image {
 		# if 0 selected, delete all reservations related to $computer_node_name
 		# Set $computer_node_name to available, proceed with questions
 		my $epoch_time = convert_to_epoch_seconds;
-		if ($chosen_request_id == 0 ) {
+		if ($chosen_request_id == 0) {
 			delete $existing_requests_array_choices{0};
 			
 			foreach my $request_id_del (sort keys %existing_requests_array_choices) {
@@ -941,7 +784,7 @@ EOF
 INSERT INTO imagerevision
 (imageid, revision, userid, datecreated, deleted, production, imagename)
 VALUES
-($image_id, '0', '$user_id', NOW( ), '1', '1', '$image_name')
+($image_id, '0', '$user_id', NOW(), '1', '1', '$image_name')
 EOF
 
 	my $imagerevision_id = database_execute($insert_imagerevision_statement);
@@ -977,7 +820,7 @@ EOF
 	print "   resource.id: $resource_id\n\n";
 	
 	
-	($request_id, $reservation_id) = insert_request($management_node_id, 'image', 'image', 0, $username, $computer_id, $image_id, $imagerevision_id, 0, 60);
+	($request_id, $reservation_id) = insert_request($management_node_id, 'image', 'image', $username, $computer_id, $image_id, $imagerevision_id, 0, 60);
 	if (!defined($request_id) || !defined($reservation_id)) {
 		print "ERROR: failed to insert new imaging request\n";
 		return;
@@ -1011,7 +854,7 @@ EOF
 		while (my $output = <COMMAND>) {
 			if ($output =~ /$reservation_id/) {
 				print $output;
-				if ($output =~ /complete/i ) {
+				if ($output =~ /complete/i) {
 					last;
 				}
 			}
@@ -1021,7 +864,7 @@ EOF
 	exit;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
