@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: new.pm 1670477 2015-03-31 20:48:08Z arkurth $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -46,7 +46,7 @@ VCL::new - Perl module for the VCL new state
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::new;
 
 # Specify the lib path using FindBin
@@ -57,7 +57,7 @@ use lib "$FindBin::Bin/..";
 use base qw(VCL::Module::State);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -69,13 +69,13 @@ use English '-no_match_vars';
 
 use VCL::utils;
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 process
 
@@ -88,7 +88,6 @@ use VCL::utils;
 sub process {
 	my $self = shift;
 	
-	my $request_data                    = $self->data->get_request_data();
 	my $request_id                      = $self->data->get_request_id();
 	my $request_state_name              = $self->data->get_request_state_name();
 	my $request_preload_only            = $self->data->get_request_preload_only();
@@ -103,7 +102,6 @@ sub process {
 	my $image_id                        = $self->data->get_image_id();
 	my $image_name                      = $self->data->get_image_name();
 	my $imagerevision_id                = $self->data->get_imagerevision_id();
-	my $user_standalone                 = $self->data->get_user_standalone();
 	
 	# If reload state is reload and computer is part of block allocation confirm imagerevisionid is the production image.
 	if ($request_state_name eq 'reload' && is_inblockrequest($computer_id)) {
@@ -350,7 +348,7 @@ sub process {
 		notify($ERRORS{'OK'}, 0, "request has been deleted, setting computer state to 'available' and exiting");
 		
 		# Update state of computer and exit
-		switch_state($request_data, '', 'available', '', '1');
+		$self->state_exit(undef, 'available');
 	}
 	
 	my $next_computer_state;
@@ -441,7 +439,7 @@ sub process {
 	exit;
 } ## end sub process
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 reload_image
 
@@ -461,76 +459,22 @@ sub reload_image {
 	my $computer_state_name             = $self->data->get_computer_state_name();
 	my $image_id                        = $self->data->get_image_id();
 	my $image_name                      = $self->data->get_image_name();
-	my $image_os_install_type           = $self->data->get_image_os_install_type();
 	my $imagerevision_id                = $self->data->get_imagerevision_id();
 	my $server_request_id               = $self->data->get_server_request_id();
 	my $server_request_fixed_ip         = $self->data->get_server_request_fixed_ip();
 	
-	# Try to get the node status if the provisioning engine has implemented a node_status() subroutine
-	my $node_status;
-	my $node_status_string = '';
-	if ($self->provisioner->can("node_status")) {
-		notify($ERRORS{'DEBUG'}, 0, "calling " . ref($self->provisioner) . "->node_status()");
-		insertloadlog($reservation_id, $computer_id, "statuscheck", "checking status of node");
-		
-		# Call node_status(), check the return value
-		$node_status = $self->provisioner->node_status();
-		
-		# Make sure a return value is defined, an error occurred if it is undefined
-		if (!defined($node_status)) {
-			notify($ERRORS{'CRITICAL'}, 0, ref($self->provisioner) . "->node_status() returned an undefined value, returning");
-			return;
-		}
-		
-		# Check what node_status returned and try to get the "status" string
-		# First see if it returned a hashref
-		if (ref($node_status) eq 'HASH') {
-			notify($ERRORS{'DEBUG'}, 0, "node_status returned a hash reference");
-			
-			# Check if the hash contains a key called "status"
-			if (defined $node_status->{status}) {
-				$node_status_string = $node_status->{status};
-				notify($ERRORS{'DEBUG'}, 0, "node_status hash reference contains key {status}=$node_status_string");
-			}
-			else {
-				notify($ERRORS{'DEBUG'}, 0, "node_status hash reference does not contain a key called 'status'");
-			}
-		} ## end if (ref($node_status) eq 'HASH')
-		
-		# Check if node_status returned an array ref
-		elsif (ref($node_status) eq 'ARRAY') {
-			notify($ERRORS{'DEBUG'}, 0, "node_status returned an array reference");
-			
-			# Check if the hash contains a key called "status"
-			if (defined((@{$node_status})[0])) {
-				$node_status_string = (@{$node_status})[0];
-				notify($ERRORS{'DEBUG'}, 0, "node_status array reference contains index [0]=$node_status_string");
-			}
-			else {
-				notify($ERRORS{'DEBUG'}, 0, "node_status array reference is empty");
-			}
-		} ## end elsif (ref($node_status) eq 'ARRAY')  [ if (ref($node_status) eq 'HASH')
-		
-		# Check if node_status didn't return a reference
-		# Assume string was returned
-		elsif (!ref($node_status)) {
-			# Use scalar value of node_status's return value
-			$node_status_string = $node_status;
-			notify($ERRORS{'DEBUG'}, 0, "node_status returned a scalar: $node_status");
-		}
-		
-		else {
-			notify($ERRORS{'CRITICAL'}, 0, ref($self->provisioner) . "->node_status() returned an unsupported reference type: " . ref($node_status) . ", returning");
-			insertloadlog($reservation_id, $computer_id, "failed", "node_status() returned an undefined value");
-			return;
-		}
-	} ## end if ($self->provisioner->can("node_status"))
+	my $node_status_string;
+	insertloadlog($reservation_id, $computer_id, "statuscheck", "checking status of node");
+	
+	# If request state is 'reinstall' or computer state is 'reload', force reload
+	if ($request_state_name eq 'reinstall' || $computer_state_name eq 'reload') {
+		$node_status_string = 'reload';
+		$computer_state_name = 'reload';
+	}
 	else {
-		notify($ERRORS{'OK'}, 0, "node status not checked, node_status() not implemented by " . ref($self->provisioner) . ", assuming load=true");
+		$node_status_string = $self->provisioner->node_status() || 'RELOAD';
 	}
 	
-	# If reinstall state - force reload state
-	$computer_state_name = 'reload' if ($request_state_name eq 'reinstall');
 	
 	if ($computer_state_name eq 'reload') {
 		# Always call load() if state is reload regardless of node_status()
@@ -544,16 +488,16 @@ sub reload_image {
 		# node_status returned 'ready'
 		notify($ERRORS{'OK'}, 0, "node_status returned '$node_status_string', $computer_short_name will not be reloaded");
 	}
-	
 	elsif ($node_status_string =~ /^post_load/i) {
 		notify($ERRORS{'OK'}, 0, "node_status returned '$node_status_string', OS post_load tasks will be performed on $computer_short_name");
 		
-		# Check if the OS module implements a post_load subroutine and that post_load has been run
+		# Check if the OS module implements a post_load subroutine
 		if ($self->os->can('post_load')) {
 			if ($self->os->post_load()) {
-				# Add the vcld_post_load line to currentimage.txt
-				$self->os->set_vcld_post_load_status();
 				$node_status_string = 'READY';
+				
+				# Add a line to currentimage.txt indicating post_load has run
+				$self->os->set_post_load_status();
 			}
 			else {
 				notify($ERRORS{'WARNING'}, 0, "failed to execute OS module's post_load() subroutine, $computer_short_name will be reloaded");
@@ -561,7 +505,8 @@ sub reload_image {
 			}
 		}
 		else {
-			notify($ERRORS{'WARNING'}, 0, "provisioning module's node_status subroutine returned '$node_status' but OS module " . ref($self->os) . " does not implement a post_load() subroutine, $computer_short_name will not be reloaded");
+			$node_status_string = 'READY';
+			notify($ERRORS{'WARNING'}, 0, "provisioning module's node_status subroutine returned '$node_status_string' but OS module " . ref($self->os) . " does not implement a post_load() subroutine, $computer_short_name will not be reloaded");
 		}
 	}
 	
@@ -577,6 +522,17 @@ sub reload_image {
 			return;
 		}
 		
+		# Update the computer state to reloading
+		if (update_computer_state($computer_id, "reloading")) {
+			notify($ERRORS{'OK'}, 0, "computer $computer_short_name state set to reloading");
+			insertloadlog($reservation_id, $computer_id, "info", "computer state updated to reloading");
+		}
+		else {
+			notify($ERRORS{'CRITICAL'}, 0, "unable to set $computer_short_name into reloading state, returning");
+			insertloadlog($reservation_id, $computer_id, "failed", "unable to set computer $computer_short_name state to reloading");
+			return;
+		}
+		
 		# Make sure the image exists on this management node's local disks
 		# Attempt to retrieve it if necessary
 		if ($self->provisioner->can("does_image_exist")) {
@@ -617,21 +573,22 @@ sub reload_image {
 			notify($ERRORS{'OK'}, 0, "unable to check if image exists, does_image_exist() not implemented by " . ref($self->provisioner));
 		}
 		
-		# Update the computer state to reloading
-		if (update_computer_state($computer_id, "reloading")) {
-			notify($ERRORS{'OK'}, 0, "computer $computer_short_name state set to reloading");
-			insertloadlog($reservation_id, $computer_id, "info", "computer state updated to reloading");
-		}
-		else {
-			notify($ERRORS{'CRITICAL'}, 0, "unable to set $computer_short_name into reloading state, returning");
-			insertloadlog($reservation_id, $computer_id, "failed", "unable to set computer $computer_short_name state to reloading");
-			return;
+		# OS currently installed on computer may not be the same type as $self->os
+		# Attempt to create a new OS object representing OS currently installed and check if that object implements a 'pre_reload' subroutine
+		if ($self->os->is_ssh_responding()) {
+			my $computer_current_os = $self->create_current_os_object($computer_id, 1);
+			if ($computer_current_os) {
+				$computer_current_os->pre_reload();
+			}
 		}
 		
 		# Call provisioning module's load() subroutine
 		notify($ERRORS{'OK'}, 0, "calling " . ref($self->provisioner) . "->load() subroutine");
 		insertloadlog($reservation_id, $computer_id, "info", "calling " . ref($self->provisioner) . "->load() subroutine");
-		if ($self->provisioner->load($node_status)) {
+		if ($self->provisioner->load()) {
+			# Add a line to currentimage.txt indicating post_load has run
+			$self->os->set_post_load_status();
+			
 			notify($ERRORS{'OK'}, 0, "$image_name was successfully reloaded on $computer_short_name");
 			insertloadlog($reservation_id, $computer_id, "loadimagecomplete", "$image_name was successfully reloaded on $computer_short_name");
 		}
@@ -666,7 +623,7 @@ sub reload_image {
 	return 1;
 } ## end sub reload_image
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 computer_not_being_used
 
@@ -688,9 +645,14 @@ sub computer_not_being_used {
 	my $image_reloadtime                = $self->data->get_image_reload_time();
 	my $request_state_name              = $self->data->get_request_state_name();
 	
-	my $attempt_limit = 5;
+	my $attempt_limit = 24;
 	ATTEMPT: for (my $attempt = 1; $attempt <= $attempt_limit; $attempt++) {
-		notify($ERRORS{'OK'}, 0, "attempt $attempt/$attempt_limit: checking for competing reservations assigned to $computer_short_name");
+		if ($attempt > 2) {
+			notify($ERRORS{'OK'}, 0, "attempt $attempt/$attempt_limit: sleeping 5 seconds before checking if $computer_short_name is not being used");
+			sleep_uninterrupted(5);
+		}
+		
+		notify($ERRORS{'OK'}, 0, "attempt $attempt/$attempt_limit: checking if $computer_short_name is not being used");
 		my $computer_state_name = $self->data->get_computer_state_name();
 		
 		# Return 0 if computer state is deleted, vmhostinuse
@@ -706,10 +668,7 @@ sub computer_not_being_used {
 			return 0;
 		}
 		
-		# Warn if computer state isn't available or reload - except for reinstall requests
-		if ($request_state_name !~ /^(reinstall)$/ && $computer_state_name !~ /^(available|reload)$/) {
-			notify($ERRORS{'WARNING'}, 0, "$computer_short_name state is $computer_state_name, checking if any conflicting reservations are active");
-		}
+		notify($ERRORS{'DEBUG'}, 0, "$computer_short_name state is $computer_state_name, checking if any competing reservations are active");
 		
 		# Check if there is another request using this machine
 		# Get a hash containing all of the reservations for the computer
@@ -717,20 +676,15 @@ sub computer_not_being_used {
 		
 		# There should be at least 1 request -- the one being processed
 		if (!$competing_request_info) {
-			notify($ERRORS{'WARNING'}, 0, "failed to retrieve any requests for computer id=$computer_id, there should be at least 1");
-			return;
-		}
-		
-		# Remove the request currently being processed from the hash
-		delete $competing_request_info->{$request_id};
-		
-		if (!keys(%$competing_request_info)) {
-			notify($ERRORS{'OK'}, 0, "$computer_short_name is not assigned to any other reservations");
-			return 1;
+			notify($ERRORS{'WARNING'}, 0, "failed to retrieve any requests for computer $computer_id, there should probably be at least 1");
+			next ATTEMPT;
 		}
 		
 		# Loop through the competing requests
 		COMPETING_REQUESTS: for my $competing_request_id (sort keys %$competing_request_info) {
+			# Ignore the request currently being processed
+			next if $competing_request_id == $request_id;
+			
 			my $competing_reservation_id    = $competing_request_info->{$competing_request_id}{data}->get_reservation_id();
 			my $competing_request_state     = $competing_request_info->{$competing_request_id}{data}->get_request_state_name();
 			my $competing_request_laststate = $competing_request_info->{$competing_request_id}{data}->get_request_laststate_name();
@@ -752,9 +706,9 @@ sub computer_not_being_used {
 			notify($ERRORS{'DEBUG'}, 0, "checking reservation assigned to $computer_short_name:\n$competing_request_info_string");
 			
 			# Check for existing image creation requests
-			if ($competing_request_state =~ /^(image)$/ || $competing_request_laststate =~ /^(image)$/) {
+			if ($competing_request_state =~ /^(image|checkpoint)$/ || $competing_request_laststate =~ /^(image|checkpoint)$/) {
 				notify($ERRORS{'WARNING'}, 0, "$computer_short_name is NOT available, it is assigned to an existing imaging reservation:\n$competing_request_info_string");
-				return 0;
+				next ATTEMPT;
 			}
 			
 			# Check for any requests in the maintenance state
@@ -777,7 +731,9 @@ sub computer_not_being_used {
 			}
 			
 			# Check if the other reservation is a 'reload' reservation for the same image revision
-			if ($competing_imagerevision_id eq $imagerevision_id && $competing_request_state =~ /^(pending|reload)$/ && $competing_request_laststate =~ /(reload)/) {
+			if ($competing_imagerevision_id eq $imagerevision_id &&
+				 $competing_request_state =~ /^(pending|reload)$/ &&
+				 $competing_request_laststate =~ /(reload)/) {
 				notify($ERRORS{'OK'}, 0, "reservation $competing_reservation_id is assigned to $computer_short_name with the same image revision: $image_name, waiting for the other reload process to complete");
 				
 				my $message = "waiting for reload reservation $competing_request_id:$competing_reservation_id to finish loading $computer_short_name with $image_name";
@@ -789,12 +745,23 @@ sub computer_not_being_used {
 				
 				# Loop until other process is done
 				if ($self->code_loop_timeout(sub{return !reservation_being_processed(@_)}, [$competing_reservation_id], $message, $total_wait_seconds, $attempt_delay_seconds)) {
-					notify($ERRORS{'DEBUG'}, 0, "reload reservation $competing_reservation_id is not loading $computer_short_name with $image_name");
-					# Verified competing 'reload' is not being processed verify it is not stuck in pending/reload
-					my ($current_competing_request_state, $current_competing_request_laststate) = get_request_current_state_name($competing_request_id);
-					if ($current_competing_request_state eq 'pending' && $current_competing_request_laststate eq 'reload') {
-						notify($ERRORS{'OK'}, 0, "state of competing reload request $competing_request_id:$competing_reservation_id is $current_competing_request_state/$current_competing_request_laststate, verified it is not being processed, changing state of competing request $competing_request_id to 'complete'");
-						update_request_state($competing_request_id, 'complete', 'reload');
+					# Check if reload request finished and was already deleted
+					if (is_request_deleted($competing_request_id)) {
+						notify($ERRORS{'OK'}, 0, "reload reservation $competing_request_id:$competing_reservation_id is no longer loading $computer_short_name with $image_name, request $competing_request_id has been deleted");
+					}
+					else {
+						# Verified competing 'reload' is not being processed verify it is not stuck in pending/reload
+						notify($ERRORS{'DEBUG'}, 0, "reload reservation $competing_request_id:$competing_reservation_id is not loading $computer_short_name with $image_name, checking current state of request $competing_request_id");
+						my ($current_competing_request_state, $current_competing_request_laststate) = get_request_current_state_name($competing_request_id);
+						if (!defined($current_competing_request_state)) {
+							if (is_request_deleted($competing_request_id)) {
+								notify($ERRORS{'OK'}, 0, "reload request $competing_request_id:$competing_reservation_id which was loading $computer_short_name with $image_name was just deleted");
+							}
+						}
+						elsif ($current_competing_request_state eq 'pending' && $current_competing_request_laststate eq 'reload') {
+							notify($ERRORS{'OK'}, 0, "state of competing reload request $competing_request_id:$competing_reservation_id is $current_competing_request_state/$current_competing_request_laststate, verified it is not being processed, changing state of competing request $competing_request_id to 'complete'");
+							update_request_state($competing_request_id, 'complete', 'reload');
+						}
 					}
 					
 					# Try again in order to retrieve a current list of competing reservations
@@ -837,25 +804,27 @@ sub computer_not_being_used {
 				if (my @competing_reservation_pids = reservation_being_processed($competing_reservation_id)) {
 					notify($ERRORS{'OK'}, 0, "reservation $competing_reservation_id is currently being processed by PID(s): " . join(', ', @competing_reservation_pids) . ", making sure the process doesn't have any Semaphore objects open before attempting to kill it");
 					
-					# Create a Semaphore object and check if the competing process owns any of its own Semaphore objects
+					# Check if the competing process owns any semaphores
 					# This would indicate it's doing something such as retrieving an image
 					# Don't kill it or a partial image may be copied
-					my $semaphore = VCL::Module::Semaphore->new();
-					for my $competing_reservation_pid (@competing_reservation_pids) {
-						if ($semaphore->get_process_semaphore_ids($competing_reservation_pid)) {
-							notify($ERRORS{'CRITICAL'}, 0, "computer $computer_short_name is NOT available, reservation $competing_reservation_id is still being processed and owns a Semaphore object, not killing the competing process, it may be transferring an image:\n$competing_request_info_string");
-							return;
+					my $semaphore_info = get_vcld_semaphore_info();
+					for my $semaphore_identifier (keys %$semaphore_info) {
+						for my $competing_reservation_pid (@competing_reservation_pids) {
+							if ($semaphore_info->{$semaphore_identifier}{reservationid} == $competing_reservation_id && $semaphore_info->{$semaphore_identifier}{pid} == $competing_reservation_pid) {
+								notify($ERRORS{'CRITICAL'}, 0, "computer $computer_short_name is NOT available, reservation $competing_reservation_id is still being processed and owns a semaphore with identifier '$semaphore_identifier', not killing the competing process, it may be transferring an image:\n$competing_request_info_string, semaphore info:\n" . format_data($semaphore_info->{$semaphore_identifier}));
+								return;
+							}
 						}
 					}
 					
-					# Kill competing process and update request state to complete
+					# Kill competing process
 					notify($ERRORS{'OK'}, 0, "attempting to kill process of competing reservation $competing_reservation_id assigned to $computer_short_name");
 					for my $competing_reservation_pid (@competing_reservation_pids) {
 						$self->mn_os->kill_process($competing_reservation_pid);
 					}
 					
 					# Wait for competing process to end before verifying that it was successfully killed
-					sleep 2;
+					sleep_uninterrupted(2);
 					
 					# Verify that the competing reservation process was killed
 					if (reservation_being_processed($competing_reservation_id)) {
@@ -864,6 +833,8 @@ sub computer_not_being_used {
 					}
 				}
 				
+				sleep_uninterrupted(5);
+				
 				# Try again in order to retrieve a current list of competing reservations
 				# The list of competing reservations may have changed
 				# A new reload reservation may have been added by timeout/deleted processes
@@ -872,11 +843,11 @@ sub computer_not_being_used {
 			}
 			elsif (reservation_being_processed($competing_reservation_id)) {
 				notify($ERRORS{'WARNING'}, 0, "computer $computer_short_name is NOT available, assigned overlapping reservations, competing reservation is currently being processed:\n$competing_request_info_string");
-				return 0;
+				next ATTEMPT;
 			}
 			else {
 				notify($ERRORS{'WARNING'}, 0, "computer $computer_short_name is NOT available, assigned overlapping reservations, competing reservation is NOT currently being processed:\n$competing_request_info_string");
-				return 0;
+				next ATTEMPT;
 			}
 		}
 		
@@ -889,7 +860,7 @@ sub computer_not_being_used {
 	return 0;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 reserve_computer
 
@@ -941,7 +912,7 @@ sub reserve_computer {
 	return 1;
 } ## end sub reserve_computer
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_child_reservations
 
@@ -962,7 +933,7 @@ sub reserve_computer {
 
 sub wait_for_child_reservations {
 	my $self              = shift;
-	my $request_data      = $self->data->get_request_data();
+	
 	my $request_id        = $self->data->get_request_id();
 	my @reservation_ids   = $self->data->get_reservation_ids();
 	my $reservation_count = $self->data->get_reservation_count();
@@ -1095,7 +1066,7 @@ sub wait_for_child_reservations {
 	return;
 } ## end sub wait_for_child_reservations
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
