@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 ###############################################################################
-# $Id: reserved.pm 1670477 2015-03-31 20:48:08Z arkurth $
+# $Id$
 ###############################################################################
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
@@ -50,7 +50,7 @@ VCL::reserved - Perl module for the VCL reserved state
 
 =cut
 
-##############################################################################
+###############################################################################
 package VCL::reserved;
 
 # Specify the lib path using FindBin
@@ -61,7 +61,7 @@ use lib "$FindBin::Bin/..";
 use base qw(VCL::Module::State);
 
 # Specify the version of this module
-our $VERSION = '2.4.2';
+our $VERSION = '2.5.1';
 
 # Specify the version of Perl to use
 use 5.008000;
@@ -73,13 +73,13 @@ use diagnostics;
 use VCL::utils;
 use POSIX qw(strftime);
 
-##############################################################################
+###############################################################################
 
 =head1 OBJECT METHODS
 
 =cut
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 process
 
@@ -94,7 +94,6 @@ sub process {
 	my $self = shift;
 	
 	my $request_id                      = $self->data->get_request_id();
-	my $request_data                    = $self->data->get_request_data();
 	my $request_logid                   = $self->data->get_request_log_id();
 	my $request_checkuser               = $self->data->get_request_checkuser();
 	my $reservation_id                  = $self->data->get_reservation_id();
@@ -103,7 +102,7 @@ sub process {
 	my $computer_short_name             = $self->data->get_computer_short_name();
 	my $is_parent_reservation           = $self->data->is_parent_reservation();
 	my $parent_reservation_id           = $self->data->get_parent_reservation_id();
-	my $server_request_id               = $self->data->get_server_request_id();
+	my $is_server_request               = $self->data->is_server_request();
 	my $imagemeta_checkuser             = $self->data->get_imagemeta_checkuser();
 	
 	my $acknowledge_timeout_seconds     = $self->os->get_timings('acknowledgetimeout');
@@ -112,6 +111,9 @@ sub process {
 	# Update the log loaded time to now for this request
 	update_log_loaded_time($request_logid);
 	
+	# Make sure firewall object is initialized early to reduce time it takes to configure things after user clicks Connect
+	$self->os->firewall() if ($self->os->can('firewall'));
+	
 	# Update the computer state to reserved
 	# This causes pending to change to the Connect button on the Current Reservations page
 	update_computer_state($computer_id, 'reserved');
@@ -121,7 +123,7 @@ sub process {
 	if ($is_parent_reservation) {
 		# Send an email and/or IM to the user
 		# Do this after updating the computer state to reserved because this is when the Connect button appears
-		$self->_notify_user_ready();
+		$self->notify_user_ready();
 		
 		# Insert acknowledgetimeout immediately before beginning to check user clicked Connect
 		# Web uses timestamp of this to determine when next to refresh the page
@@ -159,9 +161,10 @@ sub process {
 	);
 	
 	# Wait for the user to acknowledge the request by clicking Connect button or from API
+	# Note: for server requests, this will always return true because the frontend inserts reservation.remoteIP when the reservation is made
 	my $user_acknowledged = $self->code_loop_timeout(sub{$self->user_acknowledged()}, [], 'waiting for user acknowledgement', $acknowledge_timeout_remaining_seconds, 1, 10);
 	if (!$user_acknowledged) {
-		$self->_notify_user_timeout($request_data);
+		$self->notify_user_timeout_no_acknowledgement();
 		$self->state_exit('timeout', 'available', 'noack');
 	}
 	
@@ -169,19 +172,24 @@ sub process {
 	insertloadlog($reservation_id, $computer_id, "noinitialconnection", "user clicked Connect");
 	delete_computerloadlog_reservation($reservation_id, 'acknowledgetimeout');
 	
-	# The frontend should have inserted an 'initialconnecttimeout' computerloadlog entry for the parent reservation, retrieve its timestamp
-	my $connection_check_start_epoch_seconds = get_reservation_computerloadlog_time($parent_reservation_id, 'initialconnecttimeout');
-	if ($connection_check_start_epoch_seconds) {
-		notify($ERRORS{'DEBUG'}, 0, "retrieved timestamp of computerloadlog 'initialconnecttimeout' entry inserted by web frontend: $connection_check_start_epoch_seconds");
+	# For non-server requests, the frontend should have inserted an 'initialconnecttimeout' computerloadlog entry for the parent reservation when the user clicks Connect
+	# Web uses timestamp of this to determine when next to refresh the page
+	# The timestamp of this computerloadlog entry will be used to determine when to timeout the connection checking during the inuse state
+	my $connection_check_start_epoch_seconds;
+	if ($is_server_request) {
+		$connection_check_start_epoch_seconds = time;
+		insertloadlog($parent_reservation_id, $computer_id, "initialconnecttimeout", "begin initial connection timeout ($initial_connect_timeout_seconds seconds)");
 	}
 	else {
-		$connection_check_start_epoch_seconds = time;
-		notify($ERRORS{'WARNING'}, 0, "failed to retrieve timestamp of computerloadlog 'initialconnecttimeout' entry, web frontend should have inserted this, inserting new entry");
-		
-		# initialconnecttimeout should be inserted immediately after user acknowledged
-		# Web uses timestamp of this to determine when next to refresh the page
-		# The timestamp of this computerloadlog entry will be used to determine when to timeout the connection checking during the inuse state
-		insertloadlog($reservation_id, $computer_id, "initialconnecttimeout", "begin initial connection timeout ($initial_connect_timeout_seconds seconds)");
+		$connection_check_start_epoch_seconds = get_reservation_computerloadlog_time($parent_reservation_id, 'initialconnecttimeout');
+		if ($connection_check_start_epoch_seconds) {
+			notify($ERRORS{'DEBUG'}, 0, "retrieved timestamp of computerloadlog 'initialconnecttimeout' entry inserted by web frontend: $connection_check_start_epoch_seconds");
+		}
+		else {
+			notify($ERRORS{'DEBUG'}, 0, "could not retrieve timestamp of computerloadlog 'initialconnecttimeout' entry, web frontend should have inserted this, inserting new entry");
+			$connection_check_start_epoch_seconds = time;
+			insertloadlog($reservation_id, $computer_id, "initialconnecttimeout", "begin initial connection timeout ($initial_connect_timeout_seconds seconds)");
+		}
 	}
 	
 	# Call OS module's grant_access() subroutine which adds user accounts to computer
@@ -195,6 +203,12 @@ sub process {
 		$self->reservation_failed("update_cluster failed");
 	}
 	
+	# Create a JSON file containing the reservation info
+	my $enable_experimental_features = get_variable('enable_experimental_features', 0);
+	if ($enable_experimental_features) {
+		$self->os->create_reservation_info_json_file();
+	}
+	
 	# Check if OS module's post_reserve() subroutine exists
 	if ($self->os->can("post_reserve") && !$self->os->post_reserve()) {
 		$self->reservation_failed("OS module post_reserve failed");
@@ -238,30 +252,38 @@ sub process {
 		if (!$imagemeta_checkuser || !$request_checkuser) {
 			notify($ERRORS{'OK'}, 0, "never detected user connection, skipping timeout, imagemeta checkuser: $imagemeta_checkuser, request checkuser: $request_checkuser");
 		}
-		elsif ($server_request_id) {
+		elsif ($is_server_request) {
 			notify($ERRORS{'OK'}, 0, "never detected user connection, skipping timeout, server reservation");
 		}
-		elsif (is_request_deleted($request_id)) {
+		elsif (is_request_deleted($request_id) || $self->request_state_changed()) {
 			$self->state_exit();
 		}
 		else {
-			$self->_notify_user_no_login();
+			$self->notify_user_timeout_no_initial_connection();
 			$self->state_exit('timeout', 'reserved', 'nologin');
 		}
 	}
 	
+	# Add a line to currentimage.txt indicating it's possible a user logged on to the computer
+	$self->os->set_tainted_status('user may have logged in');
+	
 	# Update reservation lastcheck, otherwise inuse request will be processed immediately again
 	update_reservation_lastcheck($reservation_id);
 	
 	# Tighten up the firewall
 	# Process the connect methods again, lock the firewall down to the address the user connected from
 	my $remote_ip = $self->data->get_reservation_remote_ip();
-	if (!$self->os->process_connect_methods($remote_ip, 1)) {
-		notify($ERRORS{'CRITICAL'}, 0, "failed to process connect methods after user connected to computer");
+	if ($self->os->can('firewall') && $self->os->firewall->can('process_inuse')) {
+		$self->os->firewall->process_inuse($remote_ip);
+	}
+	else {
+		if (!$self->os->process_connect_methods($remote_ip, 1)) {
+			notify($ERRORS{'CRITICAL'}, 0, "failed to process connect methods after user connected to computer");
+		}
 	}
 	
-	# Run custom post_initial_connection scripts residing on the management node
-	$self->os->run_management_node_tools_scripts('post_initial_connection');
+	# Perform steps after a user makes an initial connection
+	$self->os->post_initial_connection();
 	
 	# For cluster reservations, the parent must wait until all child reserved processes have exited
 	# Otherwise, the state will change to inuse while the child processes are still finishing up the reserved state
@@ -280,7 +302,7 @@ sub process {
 	$self->state_exit('inuse', 'inuse');
 } ## end sub process
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 wait_for_child_reservations
 
@@ -342,7 +364,7 @@ sub wait_for_child_reservations {
 	return 1;
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 =head2 user_acknowledged
 
@@ -365,9 +387,9 @@ sub user_acknowledged {
 	
 	my $request_id = $self->data->get_request_id();
 	
-	# Check if user deleted the request
-	if (is_request_deleted($request_id)) {
-		notify($ERRORS{'DEBUG'}, 0, "request deleted, exiting");
+	# Check if the request state changed for any reason
+	# This will occur if the user deletes the request or makeproduction is initiated before the user acknowledges
+	if ($self->request_state_changed()) {
 		$self->state_exit();
 	}
 	
@@ -381,9 +403,9 @@ sub user_acknowledged {
 	}
 }
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
-=head2 _notify_user_ready
+=head2 notify_user_ready
 
  Parameters  : none
  Returns     : boolean
@@ -391,206 +413,121 @@ sub user_acknowledged {
 
 =cut
 
-sub _notify_user_ready {
+sub notify_user_ready {
 	my $self = shift;
 	
-	#my $request_id                 = $self->data->get_request_id();
-	my $request_state_name         = $self->data->get_request_id();
-	#my $reservation_id             = $self->data->get_reservation_id();
-	my $user_email                 = $self->data->get_user_email();
-	my $user_emailnotices          = $self->data->get_user_emailnotices();
-	my $user_imtype_name               = $self->data->get_user_imtype_name();
-	my $user_im_id                 = $self->data->get_user_im_id();
-	my $affiliation_sitewwwaddress = $self->data->get_user_affiliation_sitewwwaddress();
-	my $affiliation_helpaddress    = $self->data->get_user_affiliation_helpaddress();
-	my $image_prettyname           = $self->data->get_image_prettyname();
-	my $is_parent_reservation      = $self->data->is_parent_reservation();
-
-	my $mailstring;
-	my $subject;
+	my $request_state_name = $self->data->get_request_id();
+	my $user_email = $self->data->get_user_email();
+	my $user_emailnotices = $self->data->get_user_emailnotices();
+	my $user_imtype_name = $self->data->get_user_imtype_name() || 'none';;
+	my $user_im_id = $self->data->get_user_im_id();
+	my $affiliation_helpaddress = $self->data->get_user_affiliation_helpaddress();
+	my $is_parent_reservation = $self->data->is_parent_reservation();
 	
-	# Assemble the message body reservations
+	my $user_message_key;
 	if ($request_state_name =~ /^(reinstall)$/) {
-		$subject = "VCL -- $image_prettyname reservation reinstalled";
-		
-		$mailstring = <<"EOF";
-Your reservation was successfully reinstalled and you can proceed to reconnect. 
-Please revisit the 'Current Reservations' page for any additional information.
-EOF
+		$user_message_key = 'reinstalled';
 	}
 	else {
-		$subject = "VCL -- $image_prettyname reservation";
-		
-		$mailstring = <<"EOF";
-The resources for your VCL reservation have been successfully reserved.
-Connection will not be allowed until you click the 'Connect' button on the 'Current Reservations' page.
-You must acknowledge the reservation within the next 15 minutes or the resources will be reclaimed for other VCL users.
-
--Visit $affiliation_sitewwwaddress
--Select "Current Reservations"
--Click the "Connect" button
-Upon acknowledgement, all of the remaining connection details will be displayed.
-EOF
+		$user_message_key = 'reserved';
 	}
 	
-	$mailstring .= <<"EOF";
-
-Thank You,
-VCL Team
-
-******************************************************************
-This is an automated notice. If you need assistance please respond 
-with detailed information on the issue and a help ticket will be 
-generated.
-
-To disable email notices
--Visit $affiliation_sitewwwaddress
--Select User Preferences
--Select General Preferences
-
-******************************************************************
-EOF
+	my ($subject, $message) = $self->get_user_message($user_message_key);
+	if (!defined($subject) || !defined($message)) {
+		return;
+	}
 	
 	if ($is_parent_reservation && $user_emailnotices) {
-		mail($user_email, $subject, $mailstring, $affiliation_helpaddress);
+		mail($user_email, $subject, $message, $affiliation_helpaddress);
 	}
 	else {
-		# For email record keeping
-		notify($ERRORS{'MAILMASTERS'}, 0, " $user_email\n$mailstring");
+		notify($ERRORS{'MAILMASTERS'}, 0, "$user_email\n$message");
 	}
 	
 	if ($user_imtype_name ne "none") {
-		notify_via_im($user_imtype_name, $user_im_id, $mailstring, $affiliation_helpaddress);
+		notify_via_im($user_imtype_name, $user_im_id, $message, $affiliation_helpaddress);
 	}
 	
 	return 1;
-} ## end sub _notify_user_no_login
+}
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
-=head2 _notify_user_no_login
+=head2 notify_user_timeout_no_initial_connection
 
  Parameters  : none
  Returns     : boolean
- Description : Notifies the user that the request has timed out becuase no
+ Description : Notifies the user that the request has timed out because no
                initial connection was made. An e-mail and/or IM message will
                be sent to the user.
 
 =cut
 
-sub _notify_user_no_login {
+sub notify_user_timeout_no_initial_connection {
 	my $self = shift;
 	
-	my $request_id                 = $self->data->get_request_id();
-	my $reservation_id             = $self->data->get_reservation_id();
 	my $user_email                 = $self->data->get_user_email();
 	my $user_emailnotices          = $self->data->get_user_emailnotices();
-	my $user_im_name               = $self->data->get_user_imtype_name();
+	my $user_im_name               = $self->data->get_user_imtype_name() || 'none';;
 	my $user_im_id                 = $self->data->get_user_im_id();
-	my $affiliation_sitewwwaddress = $self->data->get_user_affiliation_sitewwwaddress();
 	my $affiliation_helpaddress    = $self->data->get_user_affiliation_helpaddress();
-	my $image_prettyname           = $self->data->get_image_prettyname();
 	my $is_parent_reservation      = $self->data->is_parent_reservation();
-
-	my $message = <<"EOF";
-
-Your reservation has timed out for image $image_prettyname because no initial connection was made.
-
-To make another reservation, please revisit $affiliation_sitewwwaddress.
-
-Thank You,
-VCL Team
-
-
-******************************************************************
-This is an automated notice. If you need assistance
-please respond with detailed information on the issue
-and a help ticket will be generated.
-
-To disable email notices
--Visit $affiliation_sitewwwaddress
--Select User Preferences
--Select General Preferences
-******************************************************************
-EOF
-
-	my $subject = "VCL -- Reservation Timeout";
-
+	
+	my $user_message_key = 'timeout_no_initial_connection';
+	my ($subject, $message) = $self->get_user_message($user_message_key);
+	if (!defined($subject) || !defined($message)) {
+		return;
+	}
+	
 	if ($is_parent_reservation && $user_emailnotices) {
-		#if  "0" user does not care to get additional notices
 		mail($user_email, $subject, $message, $affiliation_helpaddress);
-		notify($ERRORS{'OK'}, 0, "sent reservation timeout e-mail to $user_email");
 	}
 	if ($user_im_name ne "none") {
 		notify_via_im($user_im_name, $user_im_id, $message);
-		notify($ERRORS{'OK'}, 0, "sent reservation timeout IM to $user_im_name");
 	}
+	
 	return 1;
-} ## end sub _notify_user_no_login
+}
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
-=head2 _notify_user_timeout
+=head2 notify_user_timeout_no_acknowledgement
 
  Parameters  : none
  Returns     : boolean
- Description : Notifies the user that the request has timed out becuase no
+ Description : Notifies the user that the request has timed out because no
                initial connection was made. An e-mail and/or IM message will
                be sent to the user.
 
 =cut
 
-sub _notify_user_timeout {
+sub notify_user_timeout_no_acknowledgement {
 	my $self = shift;
 	
-	my $request_id                 = $self->data->get_request_id();
-	my $reservation_id             = $self->data->get_reservation_id();
 	my $user_email                 = $self->data->get_user_email();
 	my $user_emailnotices          = $self->data->get_user_emailnotices();
-	my $user_im_name               = $self->data->get_user_imtype_name();
+	my $user_im_name               = $self->data->get_user_imtype_name() || 'none';;
 	my $user_im_id                 = $self->data->get_user_im_id();
-	my $affiliation_sitewwwaddress = $self->data->get_user_affiliation_sitewwwaddress();
 	my $affiliation_helpaddress    = $self->data->get_user_affiliation_helpaddress();
-	my $image_prettyname           = $self->data->get_image_prettyname();
 	my $is_parent_reservation      = $self->data->is_parent_reservation();
-
-	my $message = <<"EOF";
-
-Your reservation has timed out for image $image_prettyname because no initial connection was made.
-
-To make another reservation, please revisit $affiliation_sitewwwaddress.
-
-Thank You,
-VCL Team
-
-
-******************************************************************
-This is an automated notice. If you need assistance
-please respond with detailed information on the issue
-and a help ticket will be generated.
-
-To disable email notices
--Visit $affiliation_sitewwwaddress
--Select User Preferences
--Select General Preferences
-******************************************************************
-EOF
-
-	my $subject = "VCL -- Reservation Timeout";
-
+	
+	my $user_message_key = 'timeout_no_acknowledgement';
+	my ($subject, $message) = $self->get_user_message($user_message_key);
+	if (!defined($subject) || !defined($message)) {
+		return;
+	}
+	
 	if ($is_parent_reservation && $user_emailnotices) {
-		#if  "0" user does not care to get additional notices
 		mail($user_email, $subject, $message, $affiliation_helpaddress);
-		notify($ERRORS{'OK'}, 0, "sent reservation timeout e-mail to $user_email");
 	}
 	if ($user_im_name ne "none") {
 		notify_via_im($user_im_name, $user_im_id, $message);
-		notify($ERRORS{'OK'}, 0, "sent reservation timeout IM to $user_im_name");
 	}
+	
 	return 1;
-} ## end sub _notify_user_timeout
+}
 
-#/////////////////////////////////////////////////////////////////////////////
+#//////////////////////////////////////////////////////////////////////////////
 
 1;
 __END__
