@@ -16,18 +16,19 @@
   limitations under the License.
 */
 
-# ASF VCL v2.5
-$VCLversion = '2.5';
+# ASF VCL v2.5.1
+$VCLversion = '2.5.1';
 
 require_once(".ht-inc/conf.php");
-if(! isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] != "on") {
-	header("Location: " . BASEURL . "/");
-	exit;
+
+if(SSLOFFLOAD == 0) {
+	if(! isset($_SERVER['HTTPS']) || $_SERVER['HTTPS'] != "on") {
+		header("Location: " . BASEURL . "/");
+		exit;
+	}
 }
 
-$user = '';
-$mysql_link_vcl = '';
-$mysql_link_acct = '';
+$user = array();
 $mode = '';
 $oldmode = '';
 $submitErr = '';
